/*
---------------------------------------------------------------------------
 *
 *   This file is part of OpenFTPD (based on FTP4ALL)
 *   (C) 1996-2000 by Crescent, (C) 2000 by the OpenFTPD Team
 *
 * ---------------------------------------------------------------------------
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 * ---------------------------------------------------------------------------
 *   HEADER: commondef.h - various common structs and defines
 * ---------------------------------------------------------------------------
 */

#ifndef __COMMONDEF_H__
#define __COMMONDEF_H__

#include "ac_config.h"

#include <glib.h>

/* workarounds */

/* GLIB-2.0 provides G_GINT*_FORMAT macros...
   but we have to stick with GLIB-1.2 for a while... */

#if (SIZEOF_INT == 4)
#define I32 "i"
#else
#error unsupported int size
#endif

#if (SIZEOF_LONG == 4)
#define I64 "lld"
#elif (SIZEOF_LONG == 8)
#define I64 "ld"
#else
#error unsupported int size
#endif

#define SI32 "%"I32
#define SI64 "%qi"
#define SITIME "%lu"

/********************************************
 * maximum allowed length for filenames and *
 * the whole path to a file                 *
 ********************************************/
#define OPENFTPD_NAME_MAX   128
#define OPENFTPD_PATH_MAX   512

#ifndef MAXFILENAMELENGTH
#define MAXFILENAMELENGTH   128
#endif

struct check_s
{ 
   char wildcard[8];
   char name[64];
   char param[8];
};


struct customcmd_s
{ 
   char command[16];
   char path[128];
   char param[128];
};

struct msg_s
{ 
   char* filename;
   long  filedate;
   char* msg;
   int   len;
   int   size;
   char* default_msg;
};

#define CL_NORMAL    0
#define CL_SHUTDOWN -1
#define CL_TIMEOUT  -2
#define CL_SOCKERR  -3
#define CL_KICK     -4
#define CL_PIPEERR  -5
#define CL_ERROR    -6
#define CL_FORCE    -7
#define CL_CLOSE    -8
#define CL_KILL     -9
#define CL_IDENT    -10

#define PE_CHDIR     0x01
#define PE_LIST      0x02
#define PE_MKDIR     0x04
#define PE_RMDIR     0x08
#define PE_READ      0x10
#define PE_WRITE     0x20
#define PE_OVERWRITE 0x40
#define PE_CHMOD     0x80
#define PE_ALL       0x7F
#define PE_INVISIBLE 0x01

#define FPE_READ     0x04
#define FPE_WRITE    0x02
#define FPE_CHMOD    0x08
#define FP_ALL      0x0E

#define DP_HIDDEN   0x01

#define LG_SECURITY  0x01
#define LG_TRANSFER  0x02
#define LG_FILE      0x04
#define LG_DIRECTORY 0x08
#define LG_MISC      0x10
#define LG_VERBOSE   0x20

#define DFL_REGFILE     "etc/ftpd.reg"
#define DFL_GROUPFILE   "etc/groups"
#define DFL_USERFILE    "etc/users"
#define DFL_LIMITFILE   "etc/limits.cfg"
#define DFL_LOGFILE     "var/log/ftpd.log"
#define DFL_ERRLOGFILE  "var/log/ftpd.err"
#define DFL_FTPSERRLOGFILE "var/log/ftps.err"
#define DFL_MSGDIR      "msg/"
#define DFL_MSGGDIR     "msg/grp/"
#define DFL_MSGUDIR     "msg/usr/"
#define DFL_BINDIR      "bin/"
#define DFL_BINGDIR     "bin/grp/"
#define DFL_BINUDIR     "bin/usr/"
#define DFL_SBINDIR     "sbin/"
#define DFL_SERVERPRG   "sbin/ftps"
#define DFL_ADMINPRG    "sbin/ftpa"
#define DFL_STATDIR     "var/stats/"
#define DFL_CFGDIR      "etc/"
#define DFL_PIDFILE     "var/ftpd.pid"
#define DFL_STATFILE    "var/ftpd.stats"
#define DFL_GSTATFILE   "var/ftpd.gstats"
#define DFL_SECTIONDIR  "etc/sections/"
#define DFL_SECTIONFILE "etc/sections.cfg"
#define DFL_NUKELOG     "nuke.log"
#define DFL_SECTIONSTATDIR  "etc/stats/"   /* path for the binary stat files */
#define DFL_HOSTS_ALLOW_FILE "etc/ip.allow"
#define DFL_HOSTS_DENY_FILE  "etc/ip.deny"
#define DFL_WHO_LOG     "var/who.log"
#define DFL_DUPE_LOG    "var/dupelog.bin"

#define UC_MASK      0x07
#define UC_ROOT      0x00
#define UC_NORMAL    0x01
#define UC_ANON      0x02
#define UC_TEMPLATE  0x03
#define UC_GROUPOP   0x04
#define UC_SITEOP    0x05
#define UF_MASK      0xF8
#define UF_DISABLED  0x08
#define UF_COLOR     0x10
#define UF_INVISIBLE 0x20
#define UF_NOPASSCH  0x40
#define UF_NOCREDIT  0x80
#define UL_MASK      0xFF00

#define FLAG_CHECKTIMEOUT    0
#define FLAG_CHMODALLOWED    1
#define FLAG_DESENCRYPTION   2
#define FLAG_IDENTMODE       3
#define FLAG_LOGINKICK       4
#define FLAG_NUKEBROKENUP    5
#define FLAG_PRIVATEMODE     6
#define FLAG_RESOLVER        7
#define FLAG_TOUCH           8
#define FLAG_PASSIVEMODE     9

#define FLAGS_DEFAULT 0x0296

#define ACT_IDLE             0
#define ACT_UPLOAD           1
#define ACT_DOWNLOAD         2
#define ACT_SITE_WHO         3

#define BUFFER_LEN 1024 /* common length for strings */

#endif /* __COMMONDEF_H__ */

/* EOF */
