/*
 * rfc931() speaks a common subset of the RFC 931, AUTH, TAP and IDENT
 * protocols. The code queries an RFC 931 etc. compatible daemon on a remote
 * host to look up the owner of a connection. The information should not be
 * used for authentication purposes. This routine intercepts alarm signals.
 */

#ifndef __RFC931_H__
#define __RFC931_H__

#include <glib.h>
#include <netinet/in.h>

/* rfc931 - return remote user name, given socket structures 
 * warning: returned string may be overwritten by subsequent calls
 */

/* Timeout in secs. to wait for response from server */
extern gint rfc931_timeout;

/* string to return if lookup failed */
extern const gchar *rfc931_unknown;

const gchar *rfc931(const struct sockaddr_in *rmt_sin, 
		    const struct sockaddr_in *our_sin);



#endif
