/*   ftpa.c
 *
 *   This file is part of FTP4ALL
 *
 *   Copyright (C) 1996-1999 by Crescent
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "ac_config.h"

#include <sys/types.h>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
#include "defines.h"
#include "rfc931.h"
#include "utils.h"
#include "strlcpy.h"
#include "strlcat.h"

char server_dir[256];
int  sig_array[sig_size];
int  sig_read  = 0;
int  sig_write = 0;

void signal_handler(int sig)
{ if (sig == SIGTERM) exit(CL_KILL);
  if (((sig_write==sig_size-1) && !sig_read) || (sig_write+1==sig_read))
    return;
  sig_array[sig_write] = sig;
  if (++sig_write >= sig_size) sig_write = 0;
}

int strip_colors(char* p, char* buf, size_t buf_len, char* str)
{
   if (strchr("abcdefghABCDEFGH012345678X",*(p+1))) { 
     memset(buf, 0, buf_len);
     return 1;
   }
   else {
      strlcpy(buf, "!", buf_len);
      if (*(p + 1) == '!') 
	return 1;
   }
   return 0;
}

void handle_signals(void)
{ while (sig_read != sig_write)
  { switch (sig_array[sig_read])
    { case SIGPIPE :
        exit(CL_PIPEERR);
    }
    if (++sig_read >= sig_size) sig_read = 0;
  }
}

int main(int argc, char** argv, char** env)
{ int                bouncer;
  char               buffer[4096];
  char               buff[4096];
  fd_set             fdset;
  char               ident[32];
  unsigned long      ipaddr = 0;
  struct sockaddr_in loc_addr;
  int                n = 0;
  char*              p = NULL;
  struct sockaddr_in rem_addr;
#ifndef HAVE_SIGSET
  struct sigaction   sig_act;
#endif
  int                sock = 0;
  int                timeout = 0;

  memset(ident, 0, sizeof(ident));

#ifndef HAVE_SIGSET
  sigfillset(&sig_act.sa_mask);
  sig_act.sa_flags = SA_RESTART;
#define sigset(x,y) sig_act.sa_handler = y; sigaction(x,&sig_act,NULL)
#endif
  sigset(SIGHUP,signal_handler);
  sigset(SIGINT,signal_handler);
  sigset(SIGQUIT,signal_handler);
  sigset(SIGALRM,signal_handler);
  sigset(SIGPIPE,signal_handler);
  sigset(SIGTERM,signal_handler);
  sigset(SIGUSR1,signal_handler);
  sigset(SIGUSR2,signal_handler);
  setvbuf(stdout,NULL,_IOLBF,0);

  /* evaluate environment */
  p = getenv("SOCK");
  if (!p)
    return CL_ERROR;
  sock = atoi(p);

  p = getenv("BOUNCER");
  bouncer = p ? atoi(p) : 0;

  p = getenv("BOUNCERTIMEOUT");
  timeout = p ? atoi(p) : 30;

  /* bouncer and ident check 0x01=bouncer, 0x02=ident */

  if (bouncer & 0x01) {
    /* bouncer handling, bnc4all & rftp compatible
       bnc4all syntax: AUTH [<user>]@<ip>[ <anything_else_ignored>]
       rftp syntax:    IDNT [<user>]@<ip>
    */
    n = read_to(sock,buffer,sizeof(buffer),timeout);
    if (n) {
      p = strchr(buffer,'@');
      if (p) {
	ipaddr = ntohl(inet_addr(p+1));
        *p = '\0';
        p = strchr(buffer, ' ');
        if (p)
	  strlcpy(ident,p+1,sizeof(ident));
      }
      if (!p) {
	strlcpy(buffer, "-AUTH not received\r\n", sizeof(buffer));
        send(sock,buffer,strlen(buffer),0);
      }
    }
  }
  if ((bouncer & 0x02) && !*ident)
  { n = sizeof(loc_addr);
    if (getsockname(sock,(struct sockaddr*)&loc_addr,&n) < 0) bouncer = 0;
    n = sizeof(rem_addr);
    if (getpeername(sock,(struct sockaddr*)&rem_addr,&n) < 0) bouncer = 0;
    if (bouncer)
      strlcpy(ident, rfc931(&rem_addr,&loc_addr), sizeof(ident));
  }
  if (!*ident)
    strlcpy(ident, "UNKNOWN", sizeof(ident));

  /* send ident@ip to ftpd for check, exit if check fails */
  printf("IDT_CHK %lu %s\n",ipaddr,ident);
  fgets(buffer,sizeof(buffer),stdin);
  if (*buffer != '+') return CL_IDENT;

  /* send startup message and enter command loop */
  FD_ZERO(&fdset);
  do {
     FD_SET(0,&fdset);
     FD_SET(sock,&fdset);
     n = select(sock+1,&fdset,NULL,NULL,NULL);
     if (n > 0) {
        if (FD_ISSET(0,&fdset)) {
           memset(buffer, 0, sizeof(buffer));
           n = read(0,buffer,sizeof(buffer));
           if (n > 0) { 
              p = buffer;
              while (*p) {
                 if (*p == '!' && strlen(p) > 1) {
                    if (strip_colors(p, buff + strlen(buff), sizeof(buff) - strlen(buff), buffer))
                       p++;
                 }
                 else {
                   *(buff+strlen(buff)+1) = '\0';
                   *(buff+strlen(buff)) = *p;
                 }
                 p++;
              }
              n = send(sock,buff,strlen(buff),0);
              memset(buff, 0, sizeof(buff));
              memset(buffer, 0, sizeof(buffer));
           }
        }
        if ((n > 0) && FD_ISSET(sock,&fdset)) {
           memset(buffer, 0, sizeof(buffer));
           n = recv(sock,buffer,sizeof(buffer),0);
           if (n > 0) 
              n = write(1,buffer,n);
           memset(buffer, 0, sizeof(buffer));
        }
     } 
     else {
        if (errno == EINTR) 
           handle_signals();
        else 
           return CL_ERROR;
     }
  } while (n);

  /* not reached */
  return 0;
}

/* EOF */
