/*   debug.c
 *
 *   This file is part of FTP4ALL
 *
 *   Copyright (C) 1996-1999 by Crescent
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sys/types.h>
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "externsd.h"
#include "f4adp_admin.h"
#include "ftpd.h"
#include "serverd.h"

int f4adp_dump_children(struct child_s* c, char** p, int n)
{ struct sockaddr_in addr;
  struct child_s*    ch;
  int                i;
  char               str[256];

  g_snprintf(str, sizeof(str), "+child=%p   count=%i   size=%i",
    child,children,child_size);
  server_reply(c,str);
  server_reply(c,
    "+..pid.sockfd.ident........name......ip..............bw.dr.se");
  for (i=0, ch=child; i<children; i++, ch++)
  { addr.sin_addr.s_addr = htonl(ch->ip);
    g_snprintf(str, sizeof(str), "+%5ld  %5i %-12.12s %-9s %-15s %2i %2i %2i",
      (long)ch->pid,ch->fd,ch->ident,ch->user ? ch->user->name : "???",
      inet_ntoa(addr.sin_addr),ch->lim_bw,ch->lim_direction,ch->lim_session);
    server_reply(c,str);
  }
  return 0;
}

int f4adp_dump_handler(struct child_s* c, char** p, int n)
{ struct shandler_s* h;
  char*              ha;
  int                i;
  char               str[256];
  char               tmpbuf[32];

  g_snprintf(str, sizeof(str), "+shandler=%p   count=%i   size=%i",
    shandler,shandler_count,shandler_size);
  server_reply(c,str);
  server_reply(c,"+fdno.data.....handler..............");
  for (i=0, h=shandler; i<shandler_count; i++, h++)
  { if (h->handler == child_communication) ha = "child_communication()"; else
    if (h->handler == srv_accept) ha = "srv_accept()"; else
    { g_snprintf(tmpbuf, sizeof(tmpbuf), "%p", h->handler);
      ha = tmpbuf;
    }
    g_snprintf(str, sizeof(str), "+%4i %p %s",h->fd,h->ptr,ha);
    server_reply(c,str);
  }
  return 0;
}

int f4adp_dump_watcher(struct child_s* c, char** p, int n)
{ int             i;
  char            str[256];
  struct watch_s* w;

  g_snprintf(str, sizeof(str), "+watch=%p   count=%i   size=%i   sock=%i   not_count=%i",
    watch,watch_count,watch_size,watch_sock,not_count);
  server_reply(c,str);
  server_reply(c,"+user......ipaddr...........port");
  for (i=0, w=watch; i<watch_count; i++, w++)
  { g_snprintf(str, sizeof(str), "+%-9s %-15s %5i",w->child->user->name,
      inet_ntoa(w->addr.sin_addr),(int)ntohs(w->addr.sin_port));
    server_reply(c,str);
  }
  return 0;
}

/* EOF */
