/* ---------------------------------------------------------------------------
 *
 *   This file is part of OpenFTPD (based on FTP4ALL)
 *   (C) 1996-2000 by Crescent, (C) 2000 by the OpenFTPD Team
 *
 * ---------------------------------------------------------------------------
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 * ---------------------------------------------------------------------------
 *   MODULE: f4adp_group.c - functions to work with groups
 * ---------------------------------------------------------------------------
 */

#include "ac_config.h"

#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <time.h>
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
#include "externsd.h"
#include "ftpd.h"
#include "group.h"
#include "serverd.h"
#include "user.h"
#include "utils.h"
#include "log.h"
#include "strlcpy.h"

/* group add */
int f4adp_grp_add(struct child_s* c, char** p, int n)
{ 
   struct group_s* g;
   struct group_s  grp;

   memset(&grp, 0, sizeof(grp));
   if ((strlen(p[0]) >= sizeof(grp.name)) || !sscanf(p[1], SI32, &grp.gid)) { 
      server_reply(c, "-Syntax error");
      return 0;
   }
   if (get_group_by_name(p[0])) { 
      server_reply(c,"-Group already exists");
      return 0;
   }
   if (!strcmp(p[1], "-0")) {
   for (n = 0, g = group; n < group_count; n++, g++) {
         if (g->gid >= grp.gid) grp.gid = g->gid + 1;
      }
   }
   strlcpy(grp.name, p[0], sizeof(grp.name));
   if (group_count == group_size) { 
      group_size += 8;
      g = (struct group_s*)g_realloc(group, group_size * sizeof(struct group_s));
      if (!g) { 
         group_size -= 8;
         server_reply(c,"-Out of memory");
         return 0;
      }
      group = g;
   }
   group[group_count++] = grp;
   group_dirty++;

   server_log_local(c, LG_SECURITY, "GROUP %s was added by %s", grp.name, 
      c->user->name);
   server_reply(c,"+Group added");
   return 0;
}


/* group change */
int f4adp_grp_chg(struct child_s* c, char** p, int n)
{ 
   struct group_s* g;
   struct group_s  g_new;
   gint32           i;
   int             j;
   gint64           l;
   char*           q;
   int             se = 0;
   int             pd = 0;
   gint64           r = 0;
   
   g = get_group_by_name(p[0]);
   if (!g) { 
      server_reply(c,"-Group does not exist");
      return 0;
   }
   g_new = *g;   
   for (j = 1; j < n; j++) { 
      q = strchr(p[j],'=');
      if (q) { 
         *(q++) = '\0';
         /* NAME */
	 if (!strcasecmp(p[j], "NAME")) { 
	    if (strlen(q) >= sizeof(g_new.name)) se = 1; 
	    else { 
	       if (strcasecmp(g_new.name, q)) {
	          if (get_group_by_name(q)) { 
		     server_reply(c,"-Group already exists");
                     return 0;
                  }
               }
	       strlcpy(g_new.name, q, sizeof(g_new.name));
            }
         } 
	 /* GID */
	 else if (!strcasecmp(p[j],"GID")) { 
	    if (!sscanf(q, SI32, &i)) se = 1; 
	    else {
	       if (get_group_by_gid(i)) {
	          server_reply(c,"-Group ID is already used.");
                  return 0;       
	       }
	       g_new.gid = i;
            }
	 } 
	 else if (!strcasecmp(p[j],"LIMIT")) { 
	    if (!sscanf(q,SI32,&i)) se = 1; 
	    else g_new.limit = i;
         } 
	 else if (!strcasecmp(p[j],"CREDIT")) { 
	    if (!sscanf(q,SI64,&l)) se = 1; 
	    else g_new.credit = l * 1024 * 1024;
         } 
	 else if (!strcasecmp(p[j],"DL_MAX")) { 
	    if (!sscanf(q,SI32,&i)) se = 1; 
	    else g_new.dl_speed = (g_new.dl_speed & 0xFFFF) | (i << 16);
         } 
	 else if (!strcasecmp(p[j],"DL_MIN")) { 
	    if (!sscanf(q,SI32,&i)) se = 1; 
	    else g_new.dl_speed = (g_new.dl_speed & 0xFFFF0000) | i;
         } 
	 else if (!strcasecmp(p[j],"DL_SPEED")) { 
	    if (!sscanf(q,SI32,&i)) se = 1; 
	    else g_new.dl_speed = i;
	 } 
	 else if (!strcasecmp(p[j],"UL_MAX")) { 
	     if (!sscanf(q,SI32,&i)) se = 1; 
	     else g_new.ul_speed = (g_new.ul_speed & 0xFFFF) | (i << 16);
	 } 
	 else if (!strcasecmp(p[j],"UL_MIN")) { 
	     if (!sscanf(q,SI32,&i)) se = 1; 
	     else g_new.ul_speed = (g_new.ul_speed & 0xFFFF0000) | i;
	 } 
	 else if (!strcasecmp(p[j],"UL_SPEED")) { 
	     if (!sscanf(q,SI32,&i)) se = 1; 
	     else g_new.ul_speed = i;
	 } 
	 else if (!strcasecmp(p[j],"UMASK")) { 
	    if (!sscanf(q,SI32,&i)) se = 1; 
	    else g_new.umask_group = i;
	 } 
	 else if (!strcasecmp(p[j],"LOGINS")) { 
	    if (!sscanf(q,SI32,&i)) se = 1; 
	    else g_new.logins = i;
	 }
	 else if (!strcasecmp(p[j],"MAXLOGINS")) {
	    if (!sscanf(q,SI32,&i)) se = 1; 
	    else g_new.max_logins = i;
	 }
	 else if (!strcasecmp(p[j],"SLOTS")) {
	    if (!sscanf(q,SI32,&i)) se = 1; 
	    else g_new.max_slots = i;
	 }
	 else if (!strcasecmp(p[j],"INFO")) { 
	    if (strlen(q) >= sizeof(g_new.info)) se = 1; 
	    else strlcpy(g_new.info, strcmp(q,"-") ? q : "", sizeof(g_new.info));
	 } 
	 else if (!strcasecmp(p[j],"IDENT")) { 
	    if (get_iprange(&q,g_new.ident)) se = 1;
	 } 
         /* to change group params user need R_GROUP_EDIT right.
            To change rights, he need R_USER_EDIT_RIGHTS right. 
            I think a R_GROUP_EDIT_RIGHTS is not needed          */
         
	 /* RIGHTS */
	 else if (!strcasecmp(p[j], "RIGHTS")) { 
	    if (!allowed(c->user, R_USER_EDIT_RIGHTS, NULL)) pd = 1; 
	    else if (sscanf(q, SI64, &r)) g_new.rights = r;
	    else se = 1;
	 } 
         /* RIGHTS+ */
	 else if (!strcasecmp(p[j],"RIGHTS+")) {             
            if (!allowed(c->user, R_USER_EDIT_RIGHTS, NULL)) pd =1;
            else {
               if (strcasecmp(q, "R_USER_ALL") == 0) {
                  g_new.rights |= R_USER_ALL;
               } 
	       else if (strcasecmp(q, "R_ALWAYS_LOGIN") == 0) {
		  g_new.rights |= R_ALWAYS_LOGIN;
	       }
	       else if (strcasecmp(q, "R_USER_DISABLE") == 0) {
                  g_new.rights |= R_USER_DISABLE;
               } 
	       else if (strcasecmp(q, "R_USER_KICK") == 0) {
                  g_new.rights |= R_USER_KICK;
               } 
	       else if (strcasecmp(q, "R_USER_PASSWD") == 0) {
                  g_new.rights |= R_USER_PASSWD;
               } 
	       else if (strcasecmp(q, "R_USER_ADD") == 0) {
                  g_new.rights |= R_USER_ADD;
               } 
	       else if (strcasecmp(q, "R_USER_EDIT") == 0) {
                  g_new.rights |= R_USER_EDIT;
               } 
	       else if (strcasecmp(q, "R_USER_DELETE") == 0) { 
                  g_new.rights |= R_USER_DELETE;
               } 
	       else if (strcasecmp(q, "R_USER_READ") == 0) {
                  g_new.rights |= R_USER_READ;
               } 
	       else if (strcasecmp(q, "R_USER_ADD_SO") == 0) {
                  g_new.rights |= R_USER_ADD_SO;
               } 
	       else if (strcasecmp(q, "R_USER_EDIT_SO") == 0) {
                  g_new.rights |= R_USER_EDIT_SO;
               } 
	       else if (strcasecmp(q, "R_USER_DELETE_SO") == 0) {
                  g_new.rights |= R_USER_DELETE_SO;
               } 
	       else if (strcasecmp(q, "R_USER_READ_SO") == 0) {
                  g_new.rights |= R_USER_READ_SO;
               } 
	       else if (strcasecmp(q,"R_USER_ADD_GO") == 0) {
                  g_new.rights |= R_USER_ADD_GO;
               } 
	       else if (strcasecmp(q, "R_USER_EDIT_GO") == 0) {
                  g_new.rights |= R_USER_EDIT_GO;
               } 
	       else if (strcasecmp(q, "R_USER_DELETE_GO") == 0) {
                  g_new.rights |= R_USER_DELETE_GO;
               } 
	       else if (strcasecmp(q, "R_USER_READ_GO") == 0) {
                  g_new.rights |= R_USER_READ_GO;
               } 
	       else if (strcasecmp(q, "R_USER_ADD_ANON") == 0) {
                  g_new.rights |=R_USER_ADD_ANON;
               } 
	       else if (strcasecmp(q, "R_USER_ADD_TMP") == 0) {
                  g_new.rights |=R_USER_ADD_TMP;
               }
	       else if (strcasecmp(q,"R_USER_EDIT_IP") == 0) {
                  g_new.rights |= R_USER_EDIT_IP;
               } 
	       else if (strcasecmp(q, "R_USER_EDIT_GRPS") == 0) {
                  g_new.rights |= R_USER_EDIT_GRPS;
               } 
	       else if (strcasecmp(q, "R_USER_EDIT_CREDIT") == 0) {
                  g_new.rights |= R_USER_EDIT_CREDIT;
               } 
	       else if (strcasecmp(q, "R_USER_EDIT_STATS") == 0) {
                  g_new.rights |= R_USER_EDIT_STATS;
               } 
	       else if (strcasecmp(q, "R_USER_EDIT_RIGHTS") == 0) {
                  g_new.rights |= R_USER_EDIT_RIGHTS; 
               } 
	       else if (strcasecmp(q, "R_USER_EDIT_INFO") == 0) {
                  g_new.rights |= R_USER_EDIT_INFO;
               } 
	       else if (strcasecmp(q, "R_USER_EDIT_UID") == 0) {
                  g_new.rights |= R_USER_EDIT_UID;
               }
	       else if (strcasecmp(q, "R_USER_EDIT_LOGINS") == 0) {
                  g_new.rights |= R_USER_EDIT_LOGINS;
               } 
	       else if (strcasecmp(q, "R_USER_EDIT_SPEEDS") == 0) {
                  g_new.rights |= R_USER_EDIT_SPEEDS;
               } 
	       else if (strcasecmp(q, "R_ALL") == 0) {
                  g_new.rights |= R_ALL;
               } 
	       else if (strcasecmp(q, "R_GROUP_ALL") == 0) {
                  g_new.rights |= R_GROUP_ALL;
               } 
	       else if (strcasecmp(q, "R_GROUP_ADD") == 0) {
                  g_new.rights |= R_GROUP_ADD;
               } 
	       else if (strcasecmp(q, "R_GROUP_EDIT") == 0) {
                  g_new.rights |= R_GROUP_EDIT;
               } 
	       else if (strcasecmp(q, "R_GROUP_DELETE") == 0) {
                  g_new.rights |= R_GROUP_DELETE;
               } 
	       else if (strcasecmp(q, "R_GROUP_LIST") == 0) { 
		  g_new.rights |= R_GROUP_LIST;
               } 
	       else if (strcasecmp(q, "R_NUKE") == 0) { 
		  g_new.rights |= R_NUKE;
               } 
	       else if (strcasecmp(q, "R_UNDUPE") == 0) { 
		  g_new.rights |= R_UNDUPE;
               }
               else if (strcasecmp(q,"R_STATE") == 0) {
                  g_new.rights |= R_STATE;
               }
               else if (strcasecmp(q,"R_PORTS") == 0) {
                  g_new.rights |= R_PORTS;
               }
               else if (strcasecmp(q,"R_GUI") == 0) {
                  g_new.rights |= R_GUI;
               }
               else if (strcasecmp(q,"R_MONITOR") == 0) {
                  g_new.rights |= R_MONITOR;
               }
               else if (strcasecmp(q,"R_FILEACCESS") == 0) {
                  g_new.rights |= R_FILEACCESS;
               }
               else if (strcasecmp(q,"R_CONFIG") == 0) {
                  g_new.rights |= R_CONFIG;
               }
               else if (strcasecmp(q,"R_DUMP_FILES") == 0) {
                  g_new.rights |= R_DUMP_FILES;
               }
               else if (strcasecmp(q,"R_REGISTRY") == 0) { 
                  g_new.rights |= R_REGISTRY;
               }
               else if (strcasecmp(q,"R_MESSAGES") == 0) {
                  g_new.rights |= R_MESSAGES;
               }
               else { 
		  se = 1; 
	       }
	    }
	 }
         /* RIGHTS- */
	 else if (!strcasecmp(p[j], "RIGHTS-")) {
            if (!allowed(c->user, R_USER_EDIT_RIGHTS, NULL)) pd =1;
            else {
               if (strcasecmp(q, "R_USER_ALL") == 0) {
                  g_new.rights &=~ R_USER_ALL;
               } 
	       else if (strcasecmp(q, "R_USER_DISABLE") == 0) {
                  g_new.rights &=~R_USER_DISABLE;
               }
	       else if (strcasecmp(q, "R_USER_KICK") == 0) {
                  g_new.rights &=~R_USER_KICK;
               }
	       else if (strcasecmp(q, "R_USER_PASSWD") == 0) {
                  g_new.rights &=~R_USER_PASSWD;
               }
	       else if (strcasecmp(q, "R_USER_ADD") == 0) {
                  g_new.rights &=~R_USER_ADD;
               }
	       else if (strcasecmp(q, "R_USER_EDIT") == 0) {
                  g_new.rights &=~R_USER_EDIT;
               }
	       else if (strcasecmp(q, "R_USER_DELETE") == 0) {
                  g_new.rights &=~R_USER_DELETE ;
               }
	       else if (strcasecmp(q, "R_USER_READ") == 0) {
                  g_new.rights &=~R_USER_READ;
               }
	       else if (strcasecmp(q, "R_USER_ADD_SO") == 0) {
                  g_new.rights &=~R_USER_ADD_SO;
               }
	       else if (strcasecmp(q, "R_USER_EDIT_SO") == 0) {
                  g_new.rights &=~R_USER_EDIT_SO;
               }
	       else if (strcasecmp(q, "R_USER_DELETE_SO") == 0) {
                  g_new.rights &=~R_USER_DELETE_SO;
               }
	       else if (strcasecmp(q, "R_USER_READ_SO") == 0) {
                  g_new.rights &=~R_USER_READ_SO;
               }
	       else if (strcasecmp(q, "R_USER_ADD_GO") == 0) {
                  g_new.rights &=~R_USER_ADD_GO;
               }
	       else if (strcasecmp(q, "R_USER_EDIT_GO") == 0) {
                  g_new.rights &=~R_USER_EDIT_GO;
               }
	       else if (strcasecmp(q, "R_USER_DELETE_GO") == 0) {
                  g_new.rights &=~R_USER_DELETE_GO;
               }
	       else if (strcasecmp(q, "R_USER_READ_GO") == 0) {
                  g_new.rights &=~R_USER_READ_GO;
               }
	       else if (strcasecmp(q, "R_USER_ADD_ANON") == 0) {
                  g_new.rights &=~R_USER_ADD_ANON;
               }
	       else if (strcasecmp(q, "R_USER_ADD_TMP") == 0) {
                  g_new.rights &=~R_USER_ADD_TMP;
               }
	       else if (strcasecmp(q, "R_USER_EDIT_IP") == 0) {
                  g_new.rights &=~R_USER_EDIT_IP;
               }
	       else if (strcasecmp(q, "R_USER_EDIT_GRPS") == 0) {
                  g_new.rights &=~R_USER_EDIT_GRPS;
               }
	       else if (strcasecmp(q, "R_USER_EDIT_CREDIT") == 0) {
                  g_new.rights &=~R_USER_EDIT_CREDIT;
               }
	       else if (strcasecmp(q, "R_USER_EDIT_STATS") == 0) {
                  g_new.rights &=~R_USER_EDIT_STATS;
               }
	       else if (strcasecmp(q, "R_USER_EDIT_RIGHTS") == 0) {
                  g_new.rights &=~R_USER_EDIT_RIGHTS;
               }
	       else if (strcasecmp(q, "R_USER_EDIT_INFO") == 0) {
                  g_new.rights &=~R_USER_EDIT_INFO;
               }
	       else if (strcasecmp(q, "R_USER_EDIT_UID") == 0) {
                  g_new.rights &=~R_USER_EDIT_UID;
               }
	       else if (strcasecmp(q, "R_USER_EDIT_LOGINS") == 0) {
                  g_new.rights &=~R_USER_EDIT_LOGINS;
               }
	       else if (strcasecmp(q, "R_USER_EDIT_SPEEDS") == 0) {
                  g_new.rights &=~R_USER_EDIT_SPEEDS;
               }
	       else if (strcasecmp(q, "R_ALL") == 0) {
                  g_new.rights &=~R_ALL;
               }
	       else if (strcasecmp(q, "R_GROUP_ALL") == 0) {
                  g_new.rights &=~R_GROUP_ALL;
               }
	       else if (strcasecmp(q, "R_GROUP_ADD") == 0) {
                  g_new.rights &=~R_GROUP_ADD;
               }
	       else if (strcasecmp(q, "R_GROUP_EDIT") == 0) {
                  g_new.rights &=~R_GROUP_EDIT;
               }
	       else if (strcasecmp(q, "R_GROUP_DELETE") == 0) {
                  g_new.rights &=~R_GROUP_DELETE;
               }
	       else if (strcasecmp(q, "R_GROUP_LIST") == 0) {
                  g_new.rights &=~R_GROUP_LIST;
               }
	       else if (strcasecmp(q, "R_NUKE") == 0) { 
		  g_new.rights &=~R_NUKE;
               }
	       else if (strcasecmp(q, "R_UNDUPE") == 0) { 
		  g_new.rights &=~R_UNDUPE;
               }
               else se = 1;
            }
	 }	 
	 else se = 1;
      } 
      else se = 1;
      if (se) j = n;
   }
   if (se) { 
      server_reply(c,"-Syntax error");
      return 0;
   }
   if (pd) {
      server_reply(c,"-You dont have permission");
      return 0;
   }
   *g = g_new;
   group_dirty++;
   server_log_local(c, LG_SECURITY, "GROUP %s was added by %s", g->name, 
      c->user->name);
   server_reply(c,"+Group changed");
   return 0;
}

/* group to string */
void sprintf_group(char* str, struct group_s* g)
{ 
   sprintf(str, "+%s\t"SI32"\t"SI32"\t"SI64"\t"SI32"\t"SI32"\t"SI32"\t"SI32
      "\t"SI32"\t"SI32"\t%s\t", g->name,g->gid,g->limit,g->credit,g->dl_speed,
      g->ul_speed,g->umask_group,g->logins,g->max_logins,g->max_slots,
      g->info[0] ? g->info : "-");
   print_ident(str+strlen(str), g->ident);
} 


int f4adp_grp_get(struct child_s* c, char** p, int n)
{ 
   struct group_s* g;
   char            str[256];

   g = get_group_by_name(p[0]);
   if (!g) { 
      server_reply(c,"-Group does not exist.");
      return 0;
   }
   sprintf_group(str,g);
   server_reply(c,str);
   return 0;
}


int f4adp_grp_ls(struct child_s* c, char** p, int n)
{ 
   struct group_s* g;
   char            str[256];

   for (n = 0, g = group; n < group_count; n++, g++) { 
      sprintf_group(str, g);
      server_reply(c, str);
   }
   server_reply(c, "+");
   return 0;
}


/* group list restricted */
int f4adp_grp_lsr(struct child_s* c, char** p, int n)
{ 
   struct group_s* g;
   char            str[64];

   g_snprintf(str, sizeof(str), "%i", group_count);
   server_reply(c,str);
   if (!group_count) return 0;
   if (!fgets_to(str,sizeof(str),c->rdp,5)) return 0;
   if (*str != '+') return 0;
   for (n = 0, g = group; n < group_count; n++, g++) { 
      g_snprintf(str, sizeof(str), "%s "SI32,g->name,g->gid);
      server_reply(c,str);
   }
   return 0;
}


/* group move down */
int f4adp_grp_mdn(struct child_s* c, char** p, int n)
{ 
   struct group_s* g;
   struct group_s  grp;

   if (!allowed(c->user,R_GROUP_ADD,NULL) && 
      !allowed(c->user,R_GROUP_EDIT,NULL)) { 
      server_reply(c,"-Permission denied");
      return 0;
   }
   g = get_group_by_name(p[0]);
   if (!g) { 
      server_reply(c,"-Group does not exist");
      return 0;
   }
   if (g == group+group_count - 1) { 
      server_reply(c,"-This is the last group");
      return 0;
   }
   grp    = *g;
   *g     = *(g+1);
   *(g+1) = grp;
   group_dirty++;
   server_reply(c,"+Group moved down");
   return 0;
}


/* group move up */
int f4adp_grp_mup(struct child_s* c, char** p, int n)
{ 
   struct group_s* g;
   struct group_s  grp;

   if (!allowed(c->user,R_GROUP_ADD,NULL) && 
      !allowed(c->user,R_GROUP_EDIT,NULL)) { 
      server_reply(c,"-Permission denied");
      return 0;
   }
   g = get_group_by_name(p[0]);
   if (!g) { 
      server_reply(c,"-Group does not exist");
      return 0;
   }
   if (g == group) { 
      server_reply(c,"+This is the first group");
      return 0;
   }
   grp    = *g;
   *g     = *(g-1);
   *(g-1) = grp;
   group_dirty++;
   server_reply(c,"+Group moved up");
   return 0;
}


/* group remove */
int f4adp_grp_rm(struct child_s* c, char** p, int n)
{ 
   struct group_s  *g;
   struct user_s   *u;
   int             i, j;
   char            str[256];

   g = get_group_by_name(p[0]);
   if (!g) { 
      server_reply(c,"-Group does not exist");
      return 0;
   }
   /* NEW: If group is not empty we dont remove it */
   for (i = 0; i < user_count; i++) {
      u = user + i;
      if (u->gid == g->gid) {
         g_snprintf(str, sizeof(str), "-User %s is in the group. To force remove change group"
	    " id",u->name);
         server_reply(c, str);
         return 0;
      }
      else {
         for (j = 0; j < 8; j++) {
            if (u->gids[j] == g->gid) {
               g_snprintf(str, sizeof(str), "-User %s is in the group. To force remove change"
	          " group id", u->name);
               server_reply(c, str);
               return 0;
            }
         }
      }
   } 
   memmove(g, g+1,((--group_count)-(g-group))*sizeof(struct group_s));
   group_dirty++;
   server_log_local(c, LG_SECURITY, "GROUP %s was removed by %s", p[0], 
      c->user->name);
   server_reply(c,"+Group removed");
   return 0;
}

/* group set */
int f4adp_grp_set(struct child_s* c, char** p, int n)
{ gint64           credit;
  gint32           dl_speed;
  struct group_s* g;
  gint32           gid;
  gint32           gumask;
  char            ident[sizeof(g->ident)];
  gint32           limit;
  gint32           logins;
  gint32           ul_speed;
  gint32           max_logins;
  gint32           max_slots; 

  if ((strlen(p[1]) >= sizeof(g->name)) || !sscanf(p[2],SI32,&gid)
      || !sscanf(p[3],SI32,&limit) || !sscanf(p[4],SI64,&credit)
      || !sscanf(p[5],SI32,&dl_speed) || !sscanf(p[6],SI32,&ul_speed)
      || !sscanf(p[7],SI32,&gumask) || !sscanf(p[8],SI32,&logins)
      || !sscanf(p[9],SI32,&max_logins) || !sscanf(p[10],SI32,&max_slots)
      || (strlen(p[11]) >= sizeof(g->info)) || get_iprange(p+12,ident))
  { server_reply(c,"-Syntax error");
    return 0;
  }
  g = get_group_by_name(p[0]);
  if (!g)
  { server_reply(c,"-Group does not exist");
    return 0;
  }
  if (strcasecmp(p[0],p[1])) if (get_group_by_name(p[1]))
  { server_reply(c,"-Group already exists");
    return 0;
  }
  strlcpy(g->name, p[1], sizeof(g->name));
  if (strcmp(p[2],"-0")) g->gid = gid;
  g->limit    = limit;
  g->credit   = credit;
  g->dl_speed = dl_speed;
  g->ul_speed = ul_speed;
  g->umask_group = gumask;
  g->logins   = logins;
  g->max_logins = max_logins;
  g->max_slots = max_slots; 
  strlcpy(g->info, strcmp(p[11], "-") ? p[11] : "", sizeof(g->info));
  memmove(g->ident,ident,sizeof(g->ident));
  group_dirty++;
  server_reply(c,"+Group changed");
  return 0;
}

/* EOF */
