/*   stat.c
 *                                                                              
 *   This file is part of FTP4ALL                                               
 *   
 *   Copyright (C) 1996-1999 by Crescent
 *   
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or   
 *   any later version.                                                  
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *   GNU General Public License for more details.                   
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
 */

#include "ac_config.h"

#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
#include "externsd.h"
#include "utils.h"
#include "sections.h"
#include "user.h"

struct stat_s
{ struct user_s* u;    /* user   */
  gint64          n;    /* amount */
  gint64          t;    /* time   */
} ;

struct statg_s
{ struct group_s* g;   /* group  */
  gint64           n;   /* amount */
  gint64           t;   /* time   */
} ;

struct stat_s*  s          = NULL;  /* array of users sorted by amount  */
int             stat_size  = 0;     /* capacity of array s              */
struct statg_s* sg         = NULL;  /* array of groups sorted by amount */
int             statg_size = 0;     /* capacity of array sg             */

/* g_reallocate memory for user stat array */
int alloc_stat_mem()
{ int            i;
  struct stat_s* u;

  if (stat_size < user_size)
  { i = user_size + 16;
    u = g_realloc(s,i*sizeof(struct stat_s));
    if (!u) return 0;
    s = u;
    stat_size = i;
  }
  return 1;
}

/* g_reallocate memory for group stat array */
int alloc_statg_mem()
{ struct statg_s* g;
  int             i;

  if (statg_size < group_size)
  { i = group_size + 4;
    g = g_realloc(sg,i*sizeof(struct statg_s));
    if (!g) return 0;
    sg = g;
    statg_size = i;
  }
  return 1;
}

/* reset day stats of all users */
void reset_stats_day()
{ 
   int               i, j;
   struct user_s*    u;
   struct section_s* sec;
   struct sstat_s*   st;

   for (i = 0, u = user; i < user_count; i++, u++) {
      u->dl_f_day = u->ul_f_day = u->dl_day = u->ul_day = u->dl_tm_day = 
      u->ul_tm_day = 0;
   }
   
   for (j = 0, sec = section; j < section_count; j++, sec++) {
      if ((sec) && (sec)->stat) {
         for (i = 0; i < (sec)->stat_count; i++) {
	    st = ((sec)->stat);
            if (st) {
	       (st+i)->dl_day = (st+i)->ul_day = (st+i)->dl_tm_day = 
	       (st+i)->ul_tm_day = (st+i)->ul_f_day = (st+i)->dl_f_day =0;
            }
	 }
      }
   }
   section_stat_dirty = 1;
   user_dirty = 1;
}

/* reset month stats of all users */
void reset_stats_month()
{ 
   int               i, j;
   struct user_s*    u;
   struct section_s* sec;
   struct sstat_s*   st;

   for (i = 0, u = user; i < user_count; i++, u++) {
      u->dl_f_month = u->ul_f_month = u->dl_month = u->ul_month = 
      u->dl_tm_month = u->ul_tm_month = 0;
   }

   for (j = 0, sec = section; j < section_count; j++, sec++) {
      if ((sec) && (sec)->stat) {
         for (i = 0; i < (sec)->stat_count; i++) {
	    st = ((sec)->stat);
            if (st) {
	       (st+i)->dl_month = (st+i)->ul_month = (st+i)->dl_tm_month = 
	       (st+i)->ul_tm_month = (st+i)->dl_f_month = (st+i)->ul_f_month =0;
            }
	 }
      }
   }
   section_stat_dirty = 1;
   user_dirty = 1;
}

/* reset total stats of all users */
void reset_stats_total()
{ 
   int               i, j;
   struct user_s*    u;
   struct section_s* sec;
   struct sstat_s*   st;

   for (i = 0, u = user; i < user_count; i++, u++) {
       u->dl_f_total = u->ul_f_total = u->dl_total = u->ul_total = 
       u->dl_tm_total = u->ul_tm_total = 0;
   }

   for (j = 0, sec = section; j < section_count; j++, sec++) {
      if((sec) && (sec)->stat) {
         for (i = 0; i < (sec)->stat_count; i++) {
	    st = ((sec)->stat);
            if(st) {
	       (st+i)->dl_total = (st+i)->ul_total = (st+i)->dl_tm_total = 
	       (st+i)->ul_tm_total = (st+i)->ul_f_total = (st+i)->dl_f_total =0;
            }
	 }
      }
   }
   section_stat_dirty = 1;
   user_dirty = 1;
}


/* reset week stats of all users */
void reset_stats_week()
{ 
   int               i, j;
   struct user_s*    u;
   struct section_s* sec;
   struct sstat_s*   st;

   for (i = 0, u = user; i < user_count; i++, u++) {
      u->dl_f_week = u->ul_f_week = u->dl_week = u->ul_week = u->dl_tm_week = 
      u->ul_tm_week = 0;
   }

   for (j = 0, sec = section; j < section_count; j++, sec++) {
      if ((sec) && (sec)->stat) {
         for (i = 0; i < (sec)->stat_count; i++) {
	    st = ((sec)->stat);
            if (st) {
	       (st+i)->dl_week = (st+i)->ul_week = (st+i)->dl_tm_week = 
	       (st+i)->ul_tm_week = (st+i)->dl_f_week = (st+i)->ul_f_week = 0;
            }
	 }
      }
   }
   section_stat_dirty = 1;
   user_dirty = 1;
}

/* reset year stats of all users */
void reset_stats_year()
{ 
   int               i, j;
   struct user_s*    u;
   struct section_s* sec;
   struct sstat_s*   st;

   for (i = 0, u = user; i < user_count; i++, u++) {
      u->dl_f_year = u->ul_f_year = u->dl_year = u->ul_year = u->dl_tm_year = 
      u->ul_tm_year = 0;
   }

   for (j = 0, sec = section; j < section_count; j++, sec++) {
      if((sec) && (sec)->stat) {
         for (i = 0; i < (sec)->stat_count; i++) {
	    st = ((sec)->stat);
            if (st) {
	       (st+i)->dl_year = (st+i)->ul_year = (st+i)->dl_tm_year = 
	       (st+i)->ul_tm_year = (st+i)->dl_f_year = (st+i)->ul_f_year = 0;
            }
	 }
      }
   }
   section_stat_dirty = 1;
   user_dirty = 1;
}

void init_stats()
{ struct tm* t;
  time_t     tim;

  time(&tim);
  t = localtime(&tim);
  t->tm_mon++;
  t->tm_year += 1900;
  if (t->tm_wday >= t->tm_yday) stat_week = 53;
    else stat_week = ((t->tm_yday-t->tm_wday) / 7) + 1;
  stat_month = t->tm_mon;
  stat_year = t->tm_year;
  stat_day = t->tm_yday;
}

void write_stats(time_t tim)
{ 
   int        new_month;
   int        new_day;
   int        new_week;
   int        new_year;
   struct tm* t;
   int        week;
   int        week_year;

   t = localtime(&tim);
   t->tm_mon++;
   t->tm_year += 1900;
   week_year = t->tm_year;
   if (t->tm_wday >= t->tm_yday) week_year--;

   new_year = (t->tm_year != stat_year);
   new_month = new_year || (t->tm_mon != stat_month);
   if (t->tm_wday >= t->tm_yday) week = 53;
   else week = ((t->tm_yday-t->tm_wday) / 7) + 1;
   new_week = (week != stat_week);
   new_day = (t->tm_yday != stat_day);
   if (new_day) {
      reset_stats_day();
      stat_day = t->tm_yday;
   }

   if (new_week) { 
      reset_stats_week();
      stat_week = week;
      add_weekly_credits();
   }

   if (new_month) { 
      reset_stats_month();
      stat_month = t->tm_mon;
   }

   if (new_year) { 
      reset_stats_year();
      stat_year = t->tm_year;
   }
   
   write_global_stat_file();
   write_section_stat_files();
}

/* EOF */
