/* ---------------------------------------------------------------------------
 *
 *   This file is part of OpenFTPD (based on FTP4ALL)
 *   (C) 1996-2000 by Crescent, (C) 2000 by the OpenFTPD Team
 *
 * ---------------------------------------------------------------------------
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 * ---------------------------------------------------------------------------
 *   MODULE: common.c - various functions used by multiple modules
 * ---------------------------------------------------------------------------
 */

#include "ac_config.h"

#include <sys/types.h>
#include <ctype.h>
#include <errno.h>
#include <netdb.h>
#include <signal.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include "common.h"

#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif

#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif

#ifdef HAVE_SYS_VFS_H
#include <sys/vfs.h>
#else
#include <sys/param.h>
#include <sys/mount.h>                                                     
#endif

#ifdef HAVE_STATVFS
#include <sys/statvfs.h>
/* Swap all statfs() with statvfs() */
#define statfs statvfs
#endif 

#include "externs.h"
#include "utils.h"
#include "common.h"
#include "ftps.h"
#include "log.h"
#include "perm.h"

#include "perl_layer.h"
#include "site_vfs.h"
#include "color.h"
#include "setenv.h"

#ifdef TLS
#define FPRINTF tls_fprintf
#define FPUTC   tls_fputc
#define FGETS   tls_fgets
#else
#define FPRINTF fprintf
#define FPUTC   fputc
#define FGETS   fgets
#endif

gint64
limit_ok (unsigned long size, const char *dir)
{
   int   different;
   char  str[256];

   if (superuser() || !usr.limit) return 1;
   printf("SGV_DIR different_credit %s\n",dir);
   if (!fgets(str,sizeof(str),stdin)) different = 0;
   if (*str == '-') different = 0;
   else if (sscanf(str+1, "%d", &different) != 1) different = 0;
   if (different) 
      printf("SG_CRED %s\n", dir);
   else
      printf("GT_CRED\n");
   if (!fgets(str,sizeof(str),stdin)) usr.credit = 0;
   if (sscanf(str+1, SI64, &usr.credit) != 1) usr.credit = 0;
   return (usr.credit - size);
}

int parse_command(char* p, char** param, int max, int* n)
{
  int q;

  while (skipspace(&p))
  { if (*n >= max) return 1;
    *(param+(*n)) = p;
    (*n)++;
    q = 0;
    while (!(q & 0x04)) switch (*p)
    { case '"'  :
        if (q & 0x02) p++; else
        { q ^= 0x01;
          memmove(p,p+1,strlen(p));
        }
        break;
      case '\'' :
        if (q & 0x01) p++; else
        { q ^= 0x02;
          memmove(p,p+1,strlen(p));
        }
        break;
      case '\\' :
        memmove(p,p+1,strlen(p));
        if (*p) p++;
        break;
      case '\t' :
      case ' ' :
        if (q) p++; else q |= 0x04;
        break;
      case '\0' :
        q |= 0x04;
        break;
      default :
        p++;
        break;
    }
    if (*p) *(p++) = '\0';
  }
  return 0;
}


void expand_color(char* p, char* buf, const char* str)
{ 
   if (!(usr.class & UF_COLOR)) { 
      if (*p == '!') strcpy(buf,"!"); 
      else if (strchr("abcdefghABCDEFGH012345678X",*p)) buf[0] = '\0'; 
      else { 
         strncpy(buf,p-1,2);
	 buf[2] = '\0';
      }
   } 
   else {
      switch (*p) { 
	 /* all attributes off */
	 case '0' : strcpy(buf,"\x1B[0m"); break;
	 /*  a standard ! */
	 case '!' : strcpy(buf,"!"); break;
	 /* blinking */
	 case 'X' : strcpy(buf,"\x1B[5m"); break;
	 /*********************/
	 /* FOREGROUND COLORS */
	 /*********************/
	 /* FG black */
	 case 'a' : strcpy(buf,"\x1B[0;30m"); break;
	 /* FG red */
	 case 'b' : strcpy(buf,"\x1B[0;31m"); break;
	 /* FG green */
	 case 'c' : strcpy(buf,"\x1B[0;32m"); break;
	 /* FG brown */
	 case 'd' : strcpy(buf,"\x1B[0;33m"); break;
	 /* FG blue */
	 case 'e' : strcpy(buf,"\x1B[0;34m"); break;
	 /* FG magenta */
	 case 'f' : strcpy(buf,"\x1B[0;35m"); break;
	 /* FG cyan */
	 case 'g' : strcpy(buf,"\x1B[0;36m"); break;
	 /* FG light gray */
	 case 'h' : strcpy(buf,"\x1B[0;37m"); break;
	 /* FG dark gray */
	 case 'A' : strcpy(buf,"\x1B[1;30m"); break;
	 /* FG light red */
	 case 'B' : strcpy(buf,"\x1B[1;31m"); break;
	 /* FG light green */
	 case 'C' : strcpy(buf,"\x1B[1;32m"); break;
	 /* FG yellow */
	 case 'D' : strcpy(buf,"\x1B[1;33m"); break;
	 /* FG light blue */
	 case 'E' : strcpy(buf,"\x1B[1;34m"); break;
	 /* FG light magenta */
	 case 'F' : strcpy(buf,"\x1B[1;35m"); break;
	 /* FG light cyan */
	 case 'G' : strcpy(buf,"\x1B[1;36m"); break;
	 /* FG white */
	 case 'H' : strcpy(buf,"\x1B[1;37m"); break;
	 /*********************/
	 /* BACKGROUND COLORS */
	 /*********************/
	 /* BG black */
	 case '1' : strcpy(buf,"\x1B[1;40m"); break;
	 /* BG red */
	 case '2' : strcpy(buf,"\x1B[1;41m"); break;
	 /* BG green */
	 case '3' : strcpy(buf,"\x1B[1;42m"); break;
	 /* BG brown */
	 case '4' : strcpy(buf,"\x1B[1;43m"); break;
	 /* BG blue */
	 case '5' : strcpy(buf,"\x1B[1;44m"); break;
	 /* BG magenta */
	 case '6' : strcpy(buf,"\x1B[1;45m"); break;
	 /* BG cyan */
	 case '7' : strcpy(buf,"\x1B[1;46m"); break;
	 /* BG white */
	 case '8' : strcpy(buf,"\x1B[1;47m"); break;
	 default  :
	   strncpy(buf,p-1,2);
	   buf[2] = '\0';
	   break;
      }  
   }
}


void server_expand_variable(char c, char* buf)
{ 
   printf("VAR_EXP %c\n",c);
   readline(buf, 32, stdin);
   if (*buf == '+') memmove(buf, buf+1, strlen(buf)); 
   else sprintf(buf, "%%%c", c);
}


void expand_variable(char** p, int* len, char* buf, const char* str)
{ struct sockaddr_in addr;
  struct hostent*    hp;
  int                n;
  time_t             t;
  gint64              scredit;
  struct statfs      sfs;

  switch (**p)
  { case '%' :
      strcpy(buf,"%");
      break;
    case 'a' : /* num anon */
      server_expand_variable(**p,buf);
      break;   /* max anon */
    case 'A' :
      server_expand_variable(**p,buf);
      break;

    case 'c' : /* credit */
      scredit = limit_ok(0, vfs_getcwd()); 
      if (usr.limit < 0) strcpy(buf, "upload-only"); 
      /* else if (usr.limit) sprintf(buf, SI64, usr.credit); */
      else if (usr.limit) sprintf(buf, SI64, scredit);
      else  strcpy(buf, "unlimited");
      /* sprint_bytes(buf,usr.credit,0); */
      break;

    case 'd' : /* download current session */
      /* sprint_bytes(buf,bytes_downloaded,0); */
      sprintf(buf, SI64, bytes_downloaded);
      break;
    case 'D' : /* download all sessions */
      /* sprint_bytes(buf,usr.downloaded,0); */
      sprintf(buf, SI64, usr.downloaded);
      break;
    case 'f' : /* free space */
      if (vfs_getcwd()) {
         if (statfs(strlen(vfs_getcwd()) > 1 ? vfs_getcwd()+1 : ".", &sfs) != -1) {
	    //now this will always show the number of 1024k blocks, cuz not everyone uses 1024k
            //had to do it this way to avoid overlow of the longs :)
	    sprintf(buf, "%ld", (long) ((sfs.f_bavail/1024)*sfs.f_bsize) );
	 }
	 else sprintf(buf, "%ld", (long)-1);     
      }
      else {
         sprintf(buf, "%ld", (long)-1);     
      }

      break;
    case 'h' : /* hostname */
      addr.sin_addr.s_addr = htonl(ipaddr);
      hp = gethostbyaddr((char*)&addr.sin_addr,sizeof(addr.sin_addr),
        AF_INET);
      if (hp) strcpy(buf,hp->h_name); else
        strcpy(buf,inet_ntoa(addr.sin_addr));
      break;
    case 'i' : /* ip addr */
      addr.sin_addr.s_addr = htonl(ipaddr);
      strcpy(buf,inet_ntoa(addr.sin_addr));
      break;
    case 'k' : /* last speed */
      sprintf(buf,"%3.1f",((float)last_speed)/1024);
      break;
    case 'l' : /* last login */
      strftime(buf,32,"%a %b %d, %Y %H:%M",localtime(&usr.last_login));
      break;
    case 'L' : /* limit */
      n = usr.limit;
      if (n < 0) strcpy(buf,"upload-only"); 
      else if (!n) strcpy(buf, "none"); 
      else if ((n < 100) && ((100 % n) <= 1)) sprintf(buf, "1:"SI32, 100/n); 
      else sprintf(buf, SI32, usr.limit);
      break;
    case 'n' : /* num users */
      server_expand_variable(**p, buf);
      break;
    case 'N' : /* max users */
      server_expand_variable(**p, buf);
      break;
    case 'p' : /* upload current session */
      /* sprint_bytes(buf,bytes_uploaded,0); */
      sprintf(buf, SI64, bytes_uploaded);
      break;
    case 'P' : /* download current session */
      /* sprint_bytes(buf,usr.uploaded,0); */
      sprintf(buf, SI64, usr.uploaded);
      break;
    case 'q' :
      if (!bytes_uploaded) strcpy(buf, "n/a"); 
      else sprintf(buf, "%3.1f", (float)bytes_downloaded/bytes_uploaded);
      break;
    case 'Q' :
      if (!usr.uploaded) strcpy(buf,"n/a"); else
        sprintf(buf,"%3.1f",(float)usr.downloaded/usr.uploaded);
      break;
    case 'r' :
      if (!bytes_downloaded) strcpy(buf,"n/a"); else
        sprintf(buf,"%3.1f",(float)bytes_uploaded*100/bytes_downloaded);
      break;
    case 'R' :
      if (!usr.downloaded) strcpy(buf,"n/a"); else
        sprintf(buf,"%3.1f",(float)usr.uploaded*100/usr.downloaded);
      break;
    case 's' : /* special */
      strcpy(buf,str ? str : "%s");
      break;
    case 'S' : /* username */
      strcpy(buf,usr.name);
      break;
    case 't' : /* local time */
      time(&t);
      strftime(buf,32,"%a %b %d, %Y %H:%M",localtime(&t));
      break;
    case 'u' : /* num users */
      server_expand_variable(**p,buf);
      break;
    case 'U' : /* max users */
      server_expand_variable(**p,buf);
      break;
    case 'v' : /* short version */
      strcpy(buf, VERSION);
      break;
    default :
      strncpy(buf,(*p)-1,2);
      buf[2] = '\0';
      break;
  }
}

int send_message_part(int rno, char* p, int len, const char* str, int finish,
                      int expand)
{ 
   char  buf[1024];
   int   n;
   char* param[32];

   /* we have to execute a program */
   if ((len > 3) && !strncmp(p, "%#!",3)) { 
      len -= 3;
      p += 3;
      if (*p != '/') {
         strncpy(buf,server_dir, sizeof(buf) - 1);
         strncat(buf,DFL_BINDIR, sizeof(buf) - strlen(buf) - 1);
      }
      else  strncpy(buf,"", sizeof(buf) - 1);
      while (len) { 
         if ((*p == '%') && (expand & 0x01) && (len > 1)) { 
	    len--;
            p++;
            expand_variable(&p,&len,buf+strlen(buf),str);
            if (len) { 
	       len--;
               p++;
            }
         } 
	 else if ((*p == '!') && (expand & 0x02) && (len > 1)) { 
	    len--;
            p++;
            expand_color(p,buf+strlen(buf),str);
            if (len) { 
	       len--;
               p++;
            }
         } 
	 else { 
	    switch (*p) { 
	       case '\n' : strncat(buf," ", sizeof(buf) - strlen(buf) - 1);
               case '\r' : break;
               default :
                  *(buf+strlen(buf)+1) = '\0';
                  *(buf+strlen(buf)) = *p;
            }
            len--;
            p++;
         }
      }
      n = 0;
      if (parse_command(buf,param,32,&n)) {
         FPRINTF(sockf, "%i%c%s\r\n", rno, finish ? '-' : ' ', buf);
      }
      else run_program(*param, param, n, rno, finish, vfs_getcwd()+1, &n, NULL, 0, NULL);
      return 0;
   }

   /* normal line */
   FPRINTF(sockf, "%i%c", rno, finish ? ' ' : '-');
   while (len) { 
      if ((*p == '%') && (expand & 0x01) && (len > 1)) { 
         len--;
         p++;
         expand_variable(&p, &len, buf, str);
         FPRINTF(sockf, "%s", buf);
         if (len) { 
	    len--;
            p++;
         }
      } 
      else if ((*p == '!') && (expand & 0x02) && (len > 1)) { 
         len--;
         p++;
         expand_color(p, buf, str);
         FPRINTF(sockf, "%s", buf);
         if (len) { 
	    len--;
            p++;
         }
      } 
      else { 
         switch (*p) { 
	    case '\n' :  
	       FPRINTF(sockf,"\r\n");
               break;
            case '\r' :
               break;
            default :
               FPUTC(*p, sockf);
         }
         len--;
         p++;
      }
   }
   if (*(p-1) != '\n') FPRINTF(sockf, "\r\n");
   return 0;
}

int send_message_exp(int rno, int finish, char* buffer, const char* str, int expand)
{ 
   char* p = buffer;

   /*errlog("sending <%s> <%s>",buffer, str);*/
   if (!buffer) { 
      if (finish) return -1; 
      else return 0; 
   }
   do { 
      while (*p && (*p != '\n')) p++;
      if (*p) p++;
      if (send_message_part(rno, buffer, p-buffer, str ,finish && !*p, expand)
      == -1) {
         return -1;
      }
      buffer = p;
   } 
   while (*p);
   return 0;
}

int send_message(int rno, int finish, char* buffer, const char* str)
{ 
   return send_message_exp(rno, finish, buffer, str, 3);
}

char class_char(void)
{ switch (usr.class & UC_MASK)
  { case UC_ROOT    : return 's';
    case UC_ANON    : return 'a';
    case UC_NORMAL  : return 'n';
    case UC_GROUPOP : return 'g';
    case UC_SITEOP  : return 'o';
    default         : return '-';
  }
}

void set_enviroments(char *str, int str_size, const char *relativedir)
{
   char *grp = get_groupname(usr.gid);
   char uidstr[16], gidstr[16], infostr[32];
   struct user_s* u;
   
   snprintf(uidstr, sizeof(uidstr), "%d", usr.uid);
   snprintf(gidstr, sizeof(gidstr), "%d", usr.gid);
   snprintf(infostr, sizeof(infostr), "%s", usr.info);
//   snprintf(loginstr, sizeof(loginstr), "%d", u->time);
   
#ifdef HAVE_PERL
   perl_set_var_int ("FTP::var_privileged", superuser() || groupop() || siteop());
   perl_set_var_int ("FTP::var_superuser", superuser());
   perl_set_var_int ("FTP::var_groupop", groupop());
   perl_set_var_int ("FTP::var_siteop", siteop());
   perl_set_var_str ("FTP::var_user", usr.name);
   perl_set_var_str ("FTP::var_uid", uidstr);
   perl_set_var_str ("FTP::var_gid", gidstr);
   perl_set_var_str ("FTP::var_group", grp ? grp : "unknown");
   perl_set_var_str ("FTP::var_color", coloruser() ? "1" : "0");
//   perl_set_var_str ("FTP::var_logintime", loginstr);
   perl_set_var_str ("FTP::var_tagline", infostr);
   
#endif

   setenv("F4A_UID", uidstr, 1);
   setenv("F4A_GID", gidstr, 1);
   setenv("F4A_USER", usr.name, 1);
   setenv("F4A_GROUP", grp ? grp : "unknown", 1);
   setenv("F4A_INVISIBLE", invisible() ? "i" : "-", 1);
   setenv("F4A_COLOR", coloruser() ? "c" : "-", 1);

   g_snprintf(str, str_size, "%c", class_char());
   setenv("F4A_CLASS", str, 1);

   if (anonpass) setenv("F4A_APASS", anonpass, 1); 
   setenv("F4A_STD", server_dir, 1);
   /* get user info so we can get updated one */
   printf("USR_NFO\n");
   if (!fgets(str, str_size, stdin)) 
      g_snprintf(str, str_size, "-pipe error"); 
   setenv("F4A_INFO", *str == '+' ? str + 1 : "-", 1);
   printf("RGT_CHK %s R_UNDUPE\n", usr.name);
   if (!fgets(str, str_size, stdin)) 
      g_snprintf(str, str_size, "-pipe error"); 
   setenv("F4A_R_UNDUPE", *str == '+' ? "yes" : "no", 1);
   printf("RGT_CHK %s R_NUKE\n", usr.name);
   if (!fgets(str, str_size, stdin)) 
      g_snprintf(str, str_size, "-pipe error");
   setenv("F4A_R_NUKE", *str == '+' ? "yes" : "no", 1);
   if (!relativedir) 
      printf("SG_NAME %s\n", vfs_getcwd());
   else 
      printf("SG_NAME %s\n", relativedir);
   if (!fgets(str, str_size, stdin))
      g_snprintf(str, str_size, "-pipe error");

   if (*str == '+') {
      if (str[strlen(str)-1] == '\n') str[strlen(str)-1] = '\0';
#ifdef HAVE_PERL
      perl_set_var_str ("FTP::var_section", str+1);
#endif
      setenv("F4A_SECTION", str+1, 1);
   }
   
   if (!relativedir) {
      setenv("F4A_HIDDEN", directory_flag_set_recursive(vfs_getcwd(), DP_HIDDEN) ? "yes" : "no", 1); 
      printf("SGC_DIR %s\n", vfs_getcwd());
   }
   else {
      setenv("F4A_HIDDEN", directory_flag_set_recursive(relativedir, DP_HIDDEN) ? "yes" : "no", 1); 
      printf("SGC_DIR %s\n", relativedir);
   }
   if (!fgets(str, str_size, stdin)) 
      g_snprintf(str, str_size, "-pipe error");
   if (*str == '+') {
      if (str[strlen(str)-1] == '\n') str[strlen(str)-1] = '\0';
      setenv("F4A_SECCFG", str+1, 1);
   }
}

int run_program(char* prog, char** param, int n, int rno, int finish, const char* cd,
                int* exitcode, char* buffer, int bufsize, char *relativedir)
{ 
   char*  c;
   int    err;
   FILE*  file;
   pid_t  pid;
   int    pipefd[2];
   char   str[256];
   char   str2[256];
   char   *buf;
   int    buflen;

   buflen = strlen(server_dir) + strlen(DFL_BINDIR) + strlen(prog) + 1;
   buf = g_malloc(buflen * sizeof(char));
   if (buf) 
      memset(buf, 0, buflen);
   else  
      return send_reply(550, "!BOut of memory!0");
   if (*prog != '/') {
      strncpy(buf, server_dir, buflen - 1);
      strncat(buf, DFL_BINDIR, buflen - strlen(buf) - 1);
      strncat(buf, prog, buflen - strlen(buf) - 1);
   }
   else {
      strncpy(buf, prog, buflen - 1);
   }

   err = socketpair(AF_UNIX, SOCK_STREAM, 0, pipefd);
   if (err == -1) {
     g_free(buf);
     return -1;
   }
   fflush(stdout);
   set_enviroments(str, sizeof(str), relativedir);

   pid = fork();
   if (pid == -1) {
     g_free(buf);
     return -1;
   }
   if (pid) { 
      close(pipefd[1]);
      file = fdopen(pipefd[0], "rt");
      if (!file) {
        g_free(buf);
        return -1;
      }
      if (buffer) *buffer = '\0';
      if (finish) { 
         c = fgets(str, sizeof(str)-1, file);
	 if (!c) strncpy(str, "(no output from program)", sizeof(str) - 1);
      } 
      else c = (char*)-1;
      while (c) { 
         c = fgets(str2, sizeof(str2)-1, file);
	 if (c && strlen(str2)) { 
	    if (buffer) strncat(buffer, str2, (bufsize - strlen(buffer)) - 1); 
	    else { 
	       if (finish) { 
	          if (send_message_exp(rno, 0, str, NULL, 3) == -1) {
                    g_free(buf);
                    return -1;
                  }
                  strncpy(str, str2, sizeof(str) - 1);
               } 
	       else { 
	          if (send_message_exp(rno,0,str2,NULL,3) == -1) {
                    g_free(buf);
                    return -1;
                  }
               }
            }
	 }
      }
      fclose(file);
      close(pipefd[0]);
      pid = wait(exitcode);
      g_free(buf);

      if (finish) 
	return send_reply (rno, str);

      return 0;
   } 
   else { 
      fclose(sockf);
      fclose(sockfr);
      if (cd && *cd) chdir(cd);
      close(pipefd[0]);
      dup2(pipefd[1],0);
      dup2(pipefd[1],1);
      dup2(pipefd[1],2);
      close(pipefd[1]);
      *(param+n) = NULL;
      /* execvp(prog,param); */
      execvp(buf, param);
      exit(-1);
   }
}


int run_prog_xt(char* prog, char **param, int* exitcode, char* returner, 
                char *relativedir)
{ 
   int   err;
   FILE* file;
   pid_t pid;
   int   pipefd[2];
   char  str[255];
   char* c;
   char* p;
   char  *buf;
   int   buflen;

   buflen = strlen(server_dir) + strlen(DFL_BINDIR) + strlen(prog) + 1;
   buf = g_malloc(buflen * sizeof(char));
   if (buf) 
      memset(buf, 0, buflen);
   else  
      return send_reply(550, "!BOut of memory!0");

   if (*prog != '/') {
      strncpy(buf, server_dir, buflen - 1);
      strncat(buf, DFL_BINDIR, buflen - sizeof(buf) - 1);
      strncat(buf, prog, buflen - sizeof(buf) - 1);
   }
   else {
      strncpy(buf, prog, buflen - 1);
   }

   err = socketpair(AF_UNIX,SOCK_STREAM,0,pipefd);
   if (err == -1) {
     g_free(buf);
     return -1;
   }
   fflush(stdout);
   set_enviroments(str, sizeof(str), relativedir);

   pid = fork();
   if (pid == -1) {
     g_free(buf);
     return -1;
   }
   if (pid) { 
      close(pipefd[1]);
      file = fdopen(pipefd[0],"rt");
      if (!file) {
        g_free(buf);
        return -1;
      }
      c = fgets(str,sizeof(str)-1,file);        /* get one line of the script */
      if (!c) g_snprintf(str, sizeof(str), "no special reason\n");
      if ((p = strrchr(str,'\n'))) *p = 0;
      strcpy(returner,str);  /* paste it to the calling-proc without "\n" */
      fclose(file);
      close(pipefd[0]);
      pid = wait(exitcode);
      g_free(buf);
      return 0;
   } 
   else { 
      fclose(sockf);
      fclose(sockfr);
      close(pipefd[0]);
      dup2(pipefd[1],0);
      dup2(pipefd[1],1);
      dup2(pipefd[1],2);
      close(pipefd[1]);
      *(param+2) = NULL;
      execvp(buf, param);
      g_free(buf);
      exit(-1);
   }
}

int convert_buffer(char* buffer, int len)
{ int i = 0;

  while (i < len) switch ((unsigned char)*buffer)
  { case 0xF2 :
    case 0xF4 :
    case 0xFF :
      len--;
      if (len > i) memmove(buffer,buffer+1,len-i);
      break;
    default :
      buffer++;
      i++;
  }
  return i;
}

int recvb(char* buffer, int len)
{ fd_set         fd;
  int            i;
  struct timeval t;

  FD_ZERO(&fd);
  FD_SET(fileno(sockfr),&fd);
  t.tv_sec = timeout;
  t.tv_usec = 0;
  i = select(fileno(sockfr)+1,&fd,NULL,NULL,&t);
  if (i == -1) return -1;
  if (!i)
  { quit(CL_TIMEOUT);
    errno = 0;
    return -1;
  }
  if (!FGETS(buffer,len-1,sockfr))
  { quit(CL_NORMAL);
    errno = 0;
    return -1;
  }
  return convert_buffer(buffer,strlen(buffer));
}

int read_ftp_command(char* buffer, int bufferlen, char** param, int max, int *n)
{ 
   int   len = 0;
   char* p;
   int   q;

   *n = 0;

   do { 
      if ((q = recvb(buffer + len, bufferlen - len)) == -1) return -1;
      len += q;
      if ((len >= bufferlen) && (*(buffer + len - 1) != '\n')) { 
	 do { 
            q = recvb(buffer, bufferlen);
            if (q == -1) return -1;
	 } while (*(buffer + q - 1) != '\n');

	 if (send_reply (500, "Command string too long.") == -1) 
	   return -1;
	 
	 return 1;
      }
   } while (!len && (*(buffer + len - 1) != '\n'));

   if (len && (*(buffer+len-1) == '\n')) len--;
   if (len && (*(buffer+len-1) == '\r')) len--;
   *(buffer+len) = '\0';
   /* errlog("received <%s>",buffer); */

   p = buffer; /* full command string is in p now */

   if (!skipspace(&p)) { 
     if (send_reply (500, "Command string missing command.") == -1) 
       return -1;

     return 1;
   }
   
   *param = p;
   *n = 1;
   if (skipnspace(&p)) *(p++) = '\0'; else return 0;
   if (skipspace(&p)) { 
      *(param+1) = p;
      (*n)++;
   }
   return 0;
}

#ifdef HAVE_PERL
int cat_help(char *helpstr, int n)
{
  g_return_val_if_fail (helpstr != NULL, -1);

  perl_call_subf ("FTP::cat_help", "si", helpstr, n);

  return 0;
}
#else
int cat_help(char *helpstr, int n)
{ 
   char name[256];
   FILE *file;

   mk_filename(name,"help/",NULL);
   strncat(name,helpstr, sizeof(name) - strlen(name) - 1);

   if (superuser() || groupop() || siteop()) strncat(name,".privileged", sizeof(name) - strlen(name) - 1);
   else strncat(name,".normal", sizeof(name) - strlen(name) - 1);

   if (!(file=fopen(name,"rt"))) {
      *strrchr(name,'.')=0;
      if (!(file=fopen(name,"rt"))) {
         return send_reply(501,"!BNo help available or unknown command!0");
      }
   }
   while (readline(name,sizeof(name),file)) {
      send_message(n,0,name,NULL);
   }
   send_message(n,1,"<end of help>",NULL);

   fclose(file);
   return 0;
}
#endif

/* return 0 if same day, 1 week, 2 month, 3 year, 4 nothing */
int get_time(time_t tim) 
{
   int        week;
   int        lweek;
   struct tm* t;
   struct tm* lt;
   time_t     now;

   time(&now);
   t = localtime(&tim);
   lt = localtime(&now);
   if (t->tm_wday >= t->tm_yday) week = 53;
   else week = ((t->tm_yday-t->tm_wday) / 7) + 1;
   if (t->tm_wday >= t->tm_yday) lweek = 53;
   else lweek = ((t->tm_yday-t->tm_wday) / 7) + 1;

   
   if (t->tm_yday == lt->tm_yday && t->tm_year == lt->tm_year) 
      return 0;
   if (week == lweek && t->tm_year == lt->tm_year)
      return 1;
   if (t->tm_mon == lt->tm_mon && t->tm_year == lt->tm_year)
      return 2;
   if (t->tm_year == t->tm_year)
      return 3;
   else 
      return 4;
}


int send_ftpd_reply(char *msg)
{
   char str[128];
   
   if (!fgets(str, sizeof(str), stdin)) strncpy(str, "-!Bpipe error!0", sizeof(str) - 1);
   while (*str == '+' && strcmp(str, "+\n")) {
      if (send_message(211, 0, str+1, NULL) == -1) return -1;
      if (!fgets(str, sizeof(str), stdin)) strncpy(str, "-", sizeof(str) - 1);
   }
   if (*str == '+') {
      if (msg) {
         /* if (send_message(211, 0, "<end of list>", NULL) == -1) return -1; */
         return send_reply(211, msg);
      }
      else return send_reply(211, "!H<end of list>!0");
   }
   if (strcmp(str, "-")) return send_reply(550, str+1);
   else return send_reply(211, "!Bpipe error, aborting list!0");
}


void update_user_activity(int hidden, int activity, float speed, const char *str)
{ 
   char    buf[32];

   printf("USR_ACT %d %d %.0f \"%s\"\n", hidden, activity, speed, str);
   fgets(buf, sizeof(buf), stdin);
}


void convert_filename(char* filename, int mode)
{ 
   char* name;

   if (!mode) return;
   name = filename + strlen(filename);
   while ((name > filename) && (*(name-1) != '/')) name--;
   filename = name;
   switch (mode & 0x03) { 
      case 1 :
	 while (*name) { 
	    *name = tolower(*name);
            name++;
	 }
	 break;
      
      case 2 :
	 while (*name) { 
            *name = toupper(*name);
            name++;
	 }
	 break;
      
      case 3 :
         *name = toupper(*name);
         break;
   }
   if (mode & 0x04) {
      while (*filename) { 
         if (isspace(*filename)) *filename = '_';
	 filename++;
      }
   }
}

/* EOF */
