#include "ac_config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <signal.h>
#include <time.h>
#include <dirent.h>

#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif

#include "commondef.h"

#define UC_MASK      0x07
#define UC_ROOT      0x00
#define UC_NORMAL    0x01
#define UC_GROUPOP   0x04
#define UC_SITEOP    0x05
#define UF_INVISIBLE 0x20

struct user_s
{ int   class;          /* user class                           */
  int   gid;            /* group ID                             */
  int   gids[8];        /* additional groups                    */
  char  name[10];       /* username                             */
};

struct group_s
{ int  id;
  char name[10];
};

struct msg_index
{ char   name[10];
  time_t tm; 
};

static void sendkill(char *pidf, int signal)
{  FILE *file;
   long pid;
 
   if ((file=fopen(pidf,"r"))) {
      fscanf(file,"%ld",&pid);
      fclose(file);
      kill(pid,signal);
      sleep(1);
   }
}

static void get_registry_key(char *serverdir, char *key, char *value)
{  FILE *file;
   char filename[256];
   char buffer[256];   
   char *value2 = NULL;

   g_snprintf(filename, sizeof(filename)-1, "%setc/ftpd.reg", serverdir);
   if ((file = fopen(filename, "r"))) {
      while (fgets(buffer, sizeof(buffer), file)) {
         if (!strncmp(buffer, key, strlen(key))) {
            value2 = g_realloc(value2, strlen(buffer));
            if (value2) { 
               value2 = buffer + strlen(key) + 1;
               while (*value2 == ' ' && *value2)
                  ++value2;
               if (*value2);
                  value = value2;
               if (*(value + strlen(value) - 1) == '\n')
                   *(value + strlen(value) - 1) = 0;
            }   
            break;
         }
      }
   fclose(file);
   }
}

static char *get_token(char **p, char **q, int n)
{  
   int x = 0;

   while (**q) {
      if (**q == ':' || **q == '\n') {
         x++;
         if (x == n) {
            return *p;
         }
         else
            *p = *q + 1;
      }
      (*q)++;
   }
   return NULL;
}

static int checkout_dirs(char *serverdir, char *user)
{ 
   char             dirname[256];
   DIR              *dir = NULL;

   g_snprintf(dirname, sizeof(dirname)-1, "%svar/mail", serverdir);
   dir = opendir(dirname);
   if (!dir)  
      if (mkdir(dirname, 0700) && errno != EEXIST) 
         return 0;

   g_snprintf(dirname, sizeof(dirname)-1, "%svar/mail/%s/", serverdir, user);
   dir = opendir(dirname);
   if (!dir)  
      if (mkdir(dirname, 0700) && errno != EEXIST) 
         return 0;
   g_snprintf(dirname, sizeof(dirname)-1, "%svar/mail/%s/inbox", serverdir, user);
   dir = opendir(dirname);
   if (!dir)  
      if (mkdir(dirname, 0700) && errno != EEXIST) 
         return 0;
   g_snprintf(dirname, sizeof(dirname)-1, "%svar/mail/%s/archive", serverdir, user);
   dir = opendir(dirname);
   if (!dir)  
      if (mkdir(dirname, 0700) && errno != EEXIST) 
         return 0;
   g_snprintf(dirname, sizeof(dirname)-1, "%svar/mail/%s/trash", serverdir, user);
   dir = opendir(dirname);
   if (!dir)  
      if (mkdir(dirname, 0700) && errno != EEXIST) 
         return 0;
   return 1;
}

static int siteop(struct user_s* u)
{ return u ? ((u->class & UC_MASK) == UC_SITEOP) : 0;
}

static int superuser(struct user_s* u)
{ return u ? ((u->class & UC_MASK) == UC_ROOT) : 0;
}

static int invisible(struct user_s* u)
{ return u ? (u->class & UF_INVISIBLE) : 0;
}

static struct user_s* get_user_by_name(char *name, char *serverdir)
{
   struct user_s *user = NULL;
   char          filename[256];
   FILE          *file;
   char          buffer[4092];
   char          *statfile = "var/ftpd.stats";
   char          *ptr = NULL;
   char          *p;
   char          *q;

   get_registry_key(serverdir, "statfile", statfile);

   user = g_malloc(sizeof(struct user_s));
   if (!user) return NULL;
   memset(user, 0, sizeof(struct user_s));

   g_snprintf(filename, sizeof(filename)-1, "%s%s", serverdir, statfile);
   if (!(file = fopen(filename, "r"))) return NULL;
   while (fgets(buffer, sizeof(buffer)-1, file)) {
   if (*buffer != '\n') {
      q = p = buffer;
      ptr = get_token(&p, &q, 1);
      if (!ptr) {
         fclose(file);
         return NULL;
      }     
      else if (!strncmp(p,name,q-p)) {
         fclose(file);
         strncpy(user->name, p, q-p);
          
         q = p = buffer;
         ptr = get_token(&p , &q, 3); /* gid */
         if (!ptr) return NULL;
         if (1 != sscanf(p, "%d", &user->gid)) return NULL;
         q = p = buffer;
         ptr = get_token(&p , &q, 10); /* class */
         if (!ptr) return NULL;
         if (1 != sscanf(p, "%x", &user->class)) return NULL;
         q = p = buffer;
         ptr = get_token(&p , &q, 64); /* gids */
         if (!ptr) return NULL;
         if (8 != sscanf(p, "%d,%d,%d,%d,%d,%d,%d,%d",
             &user->gids[0], &user->gids[1], &user->gids[2], &user->gids[3], 
             &user->gids[4], &user->gids[5], &user->gids[6], &user->gids[7]))
            return NULL;
         return user;
      }
   }
   }
   fclose(file);
   return NULL;
}

static struct group_s* get_group_by_name(char *name, char *serverdir)
{
   struct group_s *group = NULL;
   char           filename[256];
   FILE           *file;
   char           buffer[4092];
   char           *statfile = "etc/ftpd.gstats";
   char           *ptr = NULL;
   char           *p;
   char           *q;

   get_registry_key(serverdir, "gstatfile", statfile);

   group = g_malloc(sizeof(struct group_s));
   if (!group) return NULL;
   memset(group, 0, sizeof(struct group_s));

   g_snprintf(filename, sizeof(filename)-1, "%s%s", serverdir, statfile);
   if (!(file = fopen(filename, "r"))) return NULL;
   while (fgets(buffer, sizeof(buffer)-1, file)) {
      if (*buffer != '\n') {
         q = p = buffer;
         ptr = get_token(&p, &q, 1);
         if (!ptr) {
            fclose(file);
            return NULL;
         }     
         else if (!strncasecmp(p,name,q-p)) {
            fclose(file);
            strncpy(group->name, p, q-p);
            if (1 != sscanf(q+1, "%d", &group->id)) return NULL;
            return group;
         }
      }
   }
   fclose(file);
   return NULL;
}

int is_in_group(struct user_s *u, int id)
{
   int x;
   
   if (u->gid == id) return 1;
   for (x = 0; x < 8; x++) 
      if(u->gids[x] == id) return 1;
   return 0;
}

void cat_help()
{
   char name[256];
   FILE *file;
   char *serverdir = NULL;

   serverdir = getenv("F4A_STD");
   if (!serverdir) {
      printf("Error getting enviroments variables\n");
   } 
   g_snprintf(name, sizeof(name)-1, "%shelp/msg", serverdir);
   if (!(file=fopen(name,"rt"))) {
      printf("Error opening msg help file\n");
      return;
   }
   while (fgets(name, sizeof(name)-1, file)) {
      printf(name); 
   }
   printf("!CEnd of help!0\n");
   fclose(file);
}

int separe_nick_time(char *nick_and_t, time_t *t)
{ /* nick_and_t will carry nick and time_t the time */
   char *q = NULL;

   q = nick_and_t; 
   while (*q != '.' && *q != '\0') 
     q++;
   
   if (*q == '.') 
      *q++ = 0;
   else 
      return 0;
   
   if (sscanf(q, SITIME, t) != 1) 
      return 0;
   return 1;   
}

int cat_message(char *filename) 
{ 
   char    buff[256];
   char    name[50];
   FILE    *file;
   char    *p;
   time_t  t;
   char    str[256];

   p = strrchr(filename, '/');
   if (!p) {
      printf("Error reading msg %s\n", filename);
      return 0;
   }
   p++;
   strcpy(name,p);
   if (!separe_nick_time(name, &t)) { 
      printf("Error reading msg %s\n", p);
      return 0;
   }
   if (!(file = fopen(filename, "r"))) {
      printf("Error reading msg %s\n", filename);
      return 0;
   }
   
   strftime(buff,sizeof(buff),"%a %d/%m/%Y %H:%M:%S",localtime(&t));
   printf("\n");
   printf("!C.________________________________________________________________________.!0\n");
   printf("  !C| Message sent from: !H%-10s  !H%35s !C|!0\n", name, buff);
   printf("  !C| %-66s !C|!0\n", " ");
   while (fgets(buff, 67, file)) {
      if (*(buff+strlen(buff)-1) == '\n') *(buff+strlen(buff)-1) = 0;
      sprintf(str, "  !C| !0%-66s !C|!0\n", buff);
      printf("%s", str);
   }
   fclose(file);
   printf("!C   \\__________________________________________________!Hend of message!C__/!0\n");
   return 1;
}

void msg_read(int n, char **p)
{ 
   int              number;
   int              size;
   int              en = 0;
   int              x;
   char             dirname[256];
   char             mailfile[256];
   char             *serverdir = NULL;
   char             *username = NULL;
   char             *c = NULL;
   struct stat      st;   
   struct msg_index *msgs = NULL;
   FILE             *file;
   

   if (n == 0)
      c = "inbox";
   else if (!strcasecmp("archive", *p)) {
      p++;
      c = "archive";
   }
   else if (!strcasecmp("trash", *p)) {
      c = "trash"; /* doing this shit for not to be case sensitive */
      p++;
   }
   else if (!strcasecmp("inbox", *p)) {
      c = "inbox";
      p++; 
   }
   else
      c = "inbox";
   serverdir = getenv("F4A_STD");
   if (!serverdir) en = 1;
   username = getenv("F4A_USER");
   if (!username) en = 1;
   if (en) {
      printf("Error getting enviroments variables\n");
      return;
   }
   if (!checkout_dirs(serverdir, username)) {
      printf("!BError creating !H%s !Bmail dir, report to a siteop.!0\n", username);
      return;
   }
   g_snprintf(dirname, sizeof(dirname)-1, "%svar/mail/%s/%s/", serverdir,
            username, c);
   g_snprintf(mailfile, sizeof(mailfile)-1, "%sindex", dirname);  
   if (-1 == stat(mailfile, &st) || 0 == st.st_size) {
      printf("!CYou dont have messages in this box (!H%s!C).!0\n", c);
      return;
   }
   size = st.st_size / sizeof(struct msg_index);
   if (n > 0) {
      if (sscanf(*p, "%d", &number) != 1 || number == 0) {
         number = size;
         n = 0;
      }
   }
   else {
      number = size;
   }
   number--;
   if (number >= size) {
      printf("!CThis message number does not exist in !H%s!C folder.!0\n", c);
      return;
   }
   if (!(file = fopen(mailfile, "rt"))) {
      printf("!BCould not open your index file for this box.!0\n");
      return;
   }
   if ((msgs = g_realloc(msgs, st.st_size)) == NULL) {
      printf("Error: out of memory\n");
      return;
   }
   fread(msgs, sizeof(struct msg_index), size, file);      
   fclose(file);
   for (x = 0; x < size; x++) {
      if (x == number || n == 0) {
         g_snprintf(mailfile, sizeof(mailfile)-1, "%svar/mail/%s/%s/%s." SITIME, 
                  serverdir, username, c, (msgs + x)->name, (msgs + x)->tm);
         if (cat_message(mailfile) && !strcmp(c, "inbox")) {
         /* we move it to archive box if it is in inbox */
            g_snprintf(mailfile, sizeof(mailfile)-1, "%svar/mail/%s/archive/index",
                     serverdir, username);
            if (!(file = fopen(mailfile, "ab"))) {
               printf("!BError moving message to archive folder.!0\n");
               if (msgs) g_free(msgs);
               return;
            }
            fwrite(msgs + x, sizeof(struct msg_index), 1, file);
            fclose(file);
            g_snprintf(mailfile, sizeof(mailfile)-1, "%svar/mail/%s/archive/%s." SITIME,
                     serverdir, username, (msgs + x)->name, (msgs + x)->tm);
            g_snprintf(dirname, sizeof(dirname)-1, "%svar/mail/%s/inbox/%s." SITIME,
                     serverdir, username, (msgs + x)->name, (msgs + x)->tm);
            rename(dirname, mailfile);
            g_snprintf(mailfile, sizeof(mailfile)-1, "%svar/mail/%s/inbox/index",
                     serverdir, username);
            if (!(file = fopen(mailfile, "wb"))) {
               printf("!BError moving message to archive folder.!0\n");
               if (msgs) g_free(msgs);
               return;
            }
            memmove(msgs + x, msgs + x + 1, (size - (x + 1)) *
                    sizeof(struct msg_index));
            fwrite(msgs, sizeof(struct msg_index), size - 1, file);
            fclose(file);
            if ( x == number && n != 0) 
               x = size + 1;
            else {
               size--;
               x--;
            }
         }
      }
   }
   if (!strcmp(c, "inbox")) {
      printf("!CMessages moved to archive box.!0\n");
      if (n == 0) { /* all msg*/ 
         g_snprintf(mailfile, sizeof(mailfile)-1, "%svar/mail/%s/inbox/index",
                 serverdir, username);
         remove(mailfile);
      }
   }
   if (msgs) g_free(msgs);
}  

void write_message(char *serverdir, struct user_s *u, char *username, char *message)
{

   time_t           t;
   char             mailfile[256];
   FILE             *file;
   struct msg_index msg;

   if (!checkout_dirs(serverdir, u->name)) {
      printf("!BError creating !H%s !Bmail folders, report to a siteop.!0\n", u->name);
      return;
   }
   time(&t);
   g_snprintf(mailfile, sizeof(mailfile)-1, "%svar/mail/%s/inbox/%s." SITIME, 
            serverdir, u->name, username, t);
   if (!(file = fopen(mailfile, "w"))) {
      printf("!BError: cant send message to !H%s!0\n", u->name);
      return ;
   }
   fputs(message, file);
   fclose(file);
   g_snprintf(mailfile, sizeof(mailfile)-1, "%svar/mail/%s/inbox/index", 
            serverdir, u->name);
   if (!(file = fopen(mailfile, "ab"))) {
      printf("!BError: cant send message to !H%s!0\n", u->name);
      return ;
   }
   strcpy(msg.name,username);
   msg.tm = t;
   fwrite(&msg, sizeof(struct msg_index), 1, file);
   fclose(file);
   printf("!CMessage sent to !H%s!C.!0\n", u->name);
}

void send_to_class(int class, char *message, char *username, char *serverdir)
{
   struct user_s user;
   char          filename[256];
   FILE          *file;
   char          buffer[4092];
   char          *statfile = "var/ftpd.stats";
   char          *ptr = NULL;
   char          *p;
   char          *q;

   get_registry_key(serverdir, "statfile", statfile);
   g_snprintf(filename, sizeof(filename)-1, "%s%s", serverdir, statfile);
   if (!(file = fopen(filename, "r"))) {
      printf("!BError getting userfile.!0\n");
      return;
   }
   while (fgets(buffer, sizeof(buffer)-1, file)) {
   memset(&user,0,sizeof(struct user_s));
   if (*buffer != '\n') {
      q = p = buffer;
      ptr = get_token(&p, &q, 1);
      if (ptr) {
         strncpy(user.name, p, q-p);
         q = p = buffer;
         ptr = get_token(&p , &q, 3); /* gid */
         if (ptr) {
            if (1 == sscanf(p, "%d", &user.gid)) { 
               q = p = buffer;
               ptr = get_token(&p , &q, 10); /* class */
               if (ptr) {
                  if (1 == sscanf(p, "%x", &user.class)) {
                     if (((user.class & UC_MASK) == class) && !invisible(&user)
                        && strcmp(user.name, username))
                        write_message(serverdir, &user, username, message); 
                  }
               }
            }
         }
      }
   }
   }
   fclose(file);
   return;
}

void send_to_group(struct group_s *g, char *message, char *username, char *serverdir)
{
   struct user_s user;
   char          filename[256];
   FILE          *file;
   char          buffer[4092];
   char          *statfile = "var/ftpd.stats";
   char          *ptr = NULL;
   char          *p;
   char          *q;

   get_registry_key(serverdir, "statfile", statfile);
   g_snprintf(filename, sizeof(filename)-1, "%s%s", serverdir, statfile);
   if (!(file = fopen(filename, "r"))) {
      printf("Error getting userfile\n");
      return;
   }
   while (fgets(buffer, sizeof(buffer)-1, file)) {
   memset(&user,0,sizeof(struct user_s));
   if (*buffer != '\n') {
      q = p = buffer;
      ptr = get_token(&p, &q, 1);
      if (ptr) {
         strncpy(user.name, p, q-p);
         q = p = buffer;
         ptr = get_token(&p , &q, 3); /* gid */
         if (ptr) {
            if (1 == sscanf(p, "%d", &user.gid)) { 
               q = p = buffer;
               ptr = get_token(&p , &q, 10); /* class */
               if (ptr) {
                  if (1 == sscanf(p, "%x", &user.class)) {
                     q = p = buffer;
                     ptr = get_token(&p , &q, 64); /* gids */
                     if (ptr) {
                        if (8 == sscanf(p, "%d,%d,%d,%d,%d,%d,%d,%d",
                            &user.gids[0], &user.gids[1], &user.gids[2], 
                            &user.gids[3], &user.gids[4], &user.gids[5],
                            &user.gids[6], &user.gids[7])) {
                           if (is_in_group(&user, g->id) && !invisible(&user) &&                              strcmp(user.name, username))
                              write_message(serverdir, &user, username, message); 
                        }
                     }
                  }
               }
            }
         }
      }
   }
   }
   fclose(file);
   return;
}

void msg_list(int n, char **p)
{
   char   buff[256];
   char   dirname[256];
   char   *serverdir;
   char   *username;
   char   *c;
   int    x, size;
   int    en = 0;
   FILE   *file;
   struct msg_index *msgs = NULL;
   struct stat      st;   
   
   if (n == 0)
      c = "archive";
   else if (!strcasecmp("trash", *p)) 
      c = "trash";
   else if (!strcasecmp("inbox", *p))
      c = "inbox";
   else  
      c = "archive";
   
   p++;
   serverdir = getenv("F4A_STD");
   if (!serverdir) en = 1;
   username = getenv("F4A_USER");
   if (!username) en = 1;
   if (en) {
      printf("Error getting enviroments variables\n");
      return;
   }
   g_snprintf(dirname, sizeof(dirname)-1, "%svar/mail/%s/%s/index", serverdir,
            username, c);
   if (-1 == stat(dirname, &st) || 0 == st.st_size) {
      printf("!CYou dont have mail in this box (!H%s!C).!0\n", c);
      return;
   }
   size = st.st_size / sizeof(struct msg_index);
   if (size < 1) {
      printf("!CYou dont have mail in this box (!H%s!C).!0\n", c);
      return;
   }
   if (!(file = fopen(dirname, "rt"))) {
      printf("!BCould not open your index file for this box.!0\n");
      return;
   }
   if ((msgs = g_realloc(msgs, st.st_size)) == NULL) {
      printf("Error: out of memory\n");
      fclose(file);
      return;
   }
   fread(msgs, sizeof(struct msg_index), size, file);      
   fclose(file);

   printf("!C.________________________________________________________________________.!0\n");
   printf("  !C| %-66s !C|!0\n", " ");
   for (x = 0; x < size; x++) {
      strftime(buff, sizeof(buff), "%a %d/%m/%Y %H:%M:%S", 
               localtime(&(msgs + x)->tm));
      printf("  !C|  Message: !H%-3d  !CUser: !H%-10s  !CDate: !H%-27s !C|!0\n", x + 1, 
             (msgs + x)->name, buff);
      printf("  !C| %-66s !C|!0\n", " ");
   }
   if (!strcmp(c, "trash"))
      sprintf(buff, "__!Htrash!C__");
   else if (!strcmp(c, "archive"))
      sprintf(buff, "_!Harchive!C_");
   else
      sprintf(buff, "__!Hinbox!C__");
   printf("!C   \\_______________________________________________________%s__/!0\n", buff);
   if (msgs) g_free(msgs);
}

void msg_purge()
{
   struct dirent    *d;
   DIR              *dir = NULL;
   char             buff[256];
   char             *serverdir;
   char             *username;
   int              en = 0;

   serverdir = getenv("F4A_STD");
   if (!serverdir) en = 1;
   username = getenv("F4A_USER");
   if (!username) en = 1;
   if (en) {
      printf("Error getting enviroments variables\n");
      return;
   }
   g_snprintf(buff, sizeof(buff)-1, "%svar/mail/%s/trash/", serverdir, 
            username);
   if (!(dir = opendir(buff))) {
       printf("!BError, cant open directory trash folder.!0\n");
       return;
   }
   while ((d = readdir(dir))) {
       if (strcmp(d->d_name, ".") && strcmp(d->d_name, "..")) {
          g_snprintf(buff, sizeof(buff)-1, "%svar/mail/%s/trash/%s", serverdir,
                   username, d->d_name);
          remove(buff);
       }
   }
   printf("!CAll the messages in trash box purged.!0\n");
}

void msg_del(int n, char **p)
{
   char   buff[256];
   char   dirname[256];
   char   indexfile[256];
   char   *serverdir;
   char   *username;
   int    number, x, size;
   int    en = 0;
   FILE   *file;
   struct msg_index *msgs = NULL;
   struct stat      st;   
   
   if (n == 0) {
      printf("!BError: site msg del [number|all]!0\n");
      return;
   }
   if (!strcasecmp("all", *p))
      number = -1;
   else if (sscanf(*p, "%d", &number) != 1) {
      printf("!BError: site msg del [number|all]!0\n");
      return;
   }

   serverdir = getenv("F4A_STD");
   if (!serverdir) en = 1;
   username = getenv("F4A_USER");
   if (!username) en = 1;
   if (en) {
      printf("Error getting enviroments variables\n");
      return;
   }
   g_snprintf(dirname, sizeof(dirname)-1, "%svar/mail/%s/archive/", serverdir, 
            username);
   g_snprintf(indexfile, sizeof(indexfile)-1, "%sindex", dirname);
   if (-1 == stat(indexfile, &st) || 0 == st.st_size) {
      printf("!CYou dont have mail in archive box.!0\n");
      return;
   }
   size = st.st_size / sizeof(struct msg_index);
   if (size < 1) {
      printf("!CYou dont have mail in archive box.!0\n");
      return;
   }
   if (number > size) {
      printf("!CMessage !H%d!C doesn't exist.!0\n", number);
      return;
   }
   if (!(file = fopen(indexfile, "rt"))) {
      printf("!BCould not open your index file for archive box.!0\n");
      return;
   }
   if ((msgs = g_realloc(msgs, st.st_size)) == NULL) {
      printf("Error: out of memory\n");
      fclose(file);
      return;
   }
   fread(msgs, sizeof(struct msg_index), size, file);      
   fclose(file);
   for (x = 0; x < size; x++) {
      if (number == -1 || x + 1 == number) {
         /* we move it to trash box if it is in archive box */
         g_snprintf(buff, sizeof(buff)-1, "%svar/mail/%s/trash/index",
                  serverdir, username);
         if (!(file = fopen(buff, "ab"))) {
            printf("!BError moving message to trash folder!0\n");
            return;
         }
         fwrite(msgs + x, sizeof(struct msg_index), 1, file);
         fclose(file);
         g_snprintf(buff, sizeof(buff)-1, "%svar/mail/%s/trash/%s." SITIME,
                  serverdir, username, (msgs + x)->name, (msgs + x)->tm);
         g_snprintf(indexfile, sizeof(indexfile)-1, "%s%s." SITIME, dirname,
                  (msgs + x)->name, (msgs + x)->tm); /* old name */
         rename(indexfile, buff);
         if (number != -1) {
            g_snprintf(indexfile, sizeof(indexfile)-1, "%sindex", dirname);
            if (!(file = fopen(indexfile, "wb"))) {
               printf("!BError moving message to trash folder!0\n");
               g_snprintf(indexfile, sizeof(indexfile)-1, "%s%s." SITIME, dirname,
                        (msgs + x)->name, (msgs + x)->tm); /* old name */
               rename(buff, indexfile);
               if (msgs) g_free(msgs); 
               return;
            }
            memmove(msgs + x, msgs + x + 1, (size - (x + 1)) * 
                    sizeof(struct msg_index)); 
            fwrite(msgs, sizeof(struct msg_index), size-1, file);
            fclose(file);
            if (msgs) g_free(msgs); 
            return;
         }
      strftime(buff, sizeof(buff), "(%a %d/%m/%Y %H:%M:%S)", 
               localtime(&(msgs + x)->tm));
      printf("!CMessage from:!H %-10s %-30s !Cdeleted.!0\n", (msgs + x)->name,
             buff);
      }
   }
   if (number == -1) {
      g_snprintf(indexfile, sizeof(indexfile)-1, "%sindex", dirname);
      remove(indexfile); 
   }
   if (msgs) g_free(msgs); 
}
void msg_send(int n, char **p)
{
   int              en = 0;
   char             *serverdir = NULL;
   char             *username = NULL;
   char             *class = NULL;
   char             *invisible = NULL;
   char             message[256];
   struct user_s    *u = NULL;
   struct group_s   *g = NULL;
   int              x;

   serverdir = getenv("F4A_STD");
   if (!serverdir) en = 1;
   username = getenv("F4A_USER");
   if (!username) en = 1;
   class = getenv("F4A_CLASS");
   if (!class) en = 1;
   invisible = getenv("F4A_INVISIBLE");
   if (!invisible) en = 1;

   if (en) {
      printf("Error getting enviroments variables\n");
      return;
   }

   if (n <= 1) {
      cat_help();
      return;
   }
   *message = 0;
   for (x = 1; x < n; x++) {
      strcat(message, *(p+x));
      strcat(message, " ");
   }
   strcat(message, "\n");

   if (!strcasecmp("siteops", *p)) {
      send_to_class(UC_SITEOP, message, username, serverdir);
      send_to_class(UC_ROOT, message, username, serverdir);
   }
   else if (!strcasecmp("groupops", *p)) {
      if (*invisible != '-') {
         printf("!CAs invisible user you can only send messages to siteops.!0\n");
         return;
      }
      send_to_class(UC_GROUPOP, message, username, serverdir);
   }
   else if (!strcasecmp("admins", *p)) {
      if (*invisible != '-') {
         printf("!CAs invisible user you can only send messages to siteops.!0\n");
         return;
      }
      send_to_class(UC_SITEOP, message, username, serverdir);
      send_to_class(UC_ROOT, message, username, serverdir);
      send_to_class(UC_GROUPOP, message, username, serverdir);
   }
   else if (!strcasecmp("normal", *p)) {
      if (*invisible != '-') {
         printf("!CAs invisible user you can only send messages to siteops.!0\n");
         return;
      }
      send_to_class(UC_NORMAL, message, username, serverdir);
   }
   else if (!strcasecmp("all", *p)) {
      if (*invisible != '-') {
         printf("!CAs invisible user you can only send messages to siteops.!0\n");
         return;
      }
      send_to_class(UC_SITEOP, message, username, serverdir);
      send_to_class(UC_ROOT, message, username, serverdir);
      send_to_class(UC_GROUPOP, message, username, serverdir);
      send_to_class(UC_NORMAL, message, username, serverdir);
   }
   else {
      u = get_user_by_name(*p, serverdir);
      if (!u) {
         g = get_group_by_name(*p, serverdir);
         if (!g) {
            printf("!BSyntax error.!0\n");
            cat_help();
            return;
         }
      }
      if (u) {
         if (*invisible != '-' && !siteop(u) && !superuser(u)) {
            printf("!CAs invisible user you can only send messages to siteops.!0\n");
            return;
         }
         if (*class != 'r' && *class != 's' && *invisible != '-') {
            printf("!CUser/group does not exist.!0\n");
            return;
         }
         write_message(serverdir, u, username, message);
      }
      else if (g) {
         if (*invisible != '-') {
            printf("!CAs invisible user you can only send messages to siteops.!0\n");
            return;
         }
         send_to_group(g, message, username, serverdir);
      }
   }
}

void dump_stats_files()
{  
   char           *pidfile = "var/ftpd.pid";
   char           *serverdir;
   char           filename[256];

   serverdir = getenv("F4A_STD");
   if (!serverdir) {
      printf("Error getting enviroments variables\n");
      return;
   }
   get_registry_key(serverdir, "pidfile", pidfile);

   g_snprintf(filename, sizeof(filename)-1, "%s%s", serverdir, pidfile);
   sendkill(filename, SIGUSR2);
}

int main (int argc, char** argv, char** env)
{
   if (argc < 2) {
      cat_help();
      return 0;
   }
   argv++; 
   dump_stats_files();
   if (!strcasecmp(*argv,"read"))
      msg_read(argc-2, ++argv);  
   else if (!strcasecmp(*argv,"send")) 
      msg_send(argc-2, ++argv);
   else if (!strcasecmp(*argv,"list")) 
      msg_list(argc-2, ++argv);
   else if (!strcasecmp(*argv,"del"))  
      msg_del(argc-2, ++argv);
   else if (!strcasecmp(*argv,"purge")) 
      msg_purge();
   else
      cat_help();
   return 0;
}
