package imdb;

use LWP::Simple;
use Exporter;
use strict;

use vars qw(@ISA @EXPORT);


@ISA = qw(Exporter);

@EXPORT = qw(
	get_imdb_url
	parse_imdb
);

# get the imdb url from a text file
sub get_imdb_url {
    my $filename = shift;
    my $url = "--";
    if ( open(NFO, "< $filename") ) {
	while (<NFO>)
	{
        if ($_ =~ /(http\S+imdb.com\S+(Title|Plot)\S+\d+)/is ) {
	    $url = $1;
            if ($2 eq "Plot") {
               $url =~ s/Plot/Title/;
            }
	}
	}
	close (NFO);
    }
    return $url;
}

sub parse_imdb {
	my $url = shift;    
	my (%imdb, $content);
	unless (defined ($content = get ($url))) {
		return;
	} 
	$content = join " ", split /\n/, $content;
	$content =~ s/&#\d{1,3};/_/g;

	if ($content =~ /<TITLE>(.+)<\/TITLE>/i) {
		$imdb{'title'} = $1;
	}    
	else {
		$imdb{'title'} = "???";
	}

	if ($content =~ /Genre:<\/b>.+?<a href="\S+">(\S+)<\/a>/i) {
		$imdb{'genre'} = $1;
	}    
	else {
		$imdb{'genre'} = "???";
	}

	if ($content =~ /(Werbezeile|Tagline):<\/b> (.+) <a href=\"taglines\"/i) {
		$imdb{'tagline'} = $2;
	}
	else {
		$imdb{'tagline'} = "???";
	}
	if ($content =~ /<b>(\d\.\d)\/10<\/b> \((\S+) votes\)/) {
		$imdb{'rating'} = $1;
		$imdb{'votes'} = $2;
	}
	else {
		$imdb{'rating'} = "?";
		if ($content =~ /awaiting 5 votes/i) {
			$imdb{'votes'} = "awaiting 5";
		}
		else {
			$imdb{'votes'} = "?";
		}
	}

	if ($content =~ /(Plot (Outline|Summary)|Kurzbeschreibung|Inhaltsangabe):<\/b> (.+?)(<a href=\"plotsummary\">)/i ) {
		my $plot = $3;
		$plot =~ s/<a href=\".*\">//;
		$plot =~ s/<\/a>//;
		$imdb{'plot'} = $plot;
	}
	elsif ($content =~ /(Plot (Outline|Summary)|Kurzbeschreibung|Inhaltsangabe):<\/b> (.+?)(<a href=\"trailers\">)/i ) {
		my $plot = $3;
		$plot =~ s/<a href=\".*\">//;
		$plot =~ s/<\/a>//;
		$imdb{'plot'} = $plot;
	}
	else {
		$imdb{'plot'} = "???";
	}

	return %imdb;
}

