##############################################################################
#                                                                            #
#  mp3info v1.2 (c) 2001 pedro                                               #
#                                                                            #
#  returns a hash with all id3tags and header infos from mp3                 #
#  (this is a patched and updated version of freddies zipscript)             #
# ---------------------------------------------------------------------------#
#                                                                            #
#  This program is free software; you can redistribute it and/or modify      #
#  it under the terms of the GNU General Public License as published by      #
#  the Free Software Foundation; either version 2 of the License, or         #
#  (at your option) any later version.                                       #
#                                                                            #
##############################################################################

package mp3info;


use strict; #always
require Exporter;

use vars qw (@ISA @EXPORT);
@ISA = qw(Exporter);

@EXPORT = qw(
	mp3_info
);

# ----------------------------------------------------------------------------
#   bin2dec() - convert a binary string to an int
# ----------------------------------------------------------------------------
sub bin2dec ($) {
    return unpack("N", pack("B32", substr("0" x 32 . shift, -32)));
}

# ----------------------------------------------------------------------------
#   id3v2_size() - unmangle the total size of the ID3v2 header
# ----------------------------------------------------------------------------
sub id3v2_size ($) {
	my $id3size = shift;
	my $newsize = 0;
	for my $i (0..3) {
		my $mask = 0xff << ($i * 8);
		my $val = ($id3size & $mask ) >> ($i * 8);
		$newsize |= $val << ($i * 7);
	}
	return $newsize;
}

# ----------------------------------------------------------------------------
#   mp3_info($filename) - retrieve info about an mp3
# ----------------------------------------------------------------------------
sub mp3_info  {
	my $filename = shift;
	my %mp3info = ();
	my @mpeg = (2, 1);
	my @layer = ('bad', 3, 2, 1);
	my %bitrate = (
	    '1:1' => [ 'bad', 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448, 'bad' ],
	    '1:2' => [ 'bad', 32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384, 'bad' ],
	    '1:3' => [ 'bad', 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 'bad' ],
	    '2:1' => [ 'bad', 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256, 'bad' ],
	    '2:2' => [ 'bad', 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 'bad' ],
	    '2:3' => [ 'bad', 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 'bad' ],

	);
	my @frequency = ( 44100, 48000, 32000 );
	my @mode = ('Stereo', 'Joint stereo', 'Dual channel', 'Mono');
	my $filesize = -s $filename;
	my $length = 0;
	# ID3 genres
	my @genres = ('Blues','Classic Rock','Country','Dance','Disco','Funk','Grunge',
		      'Hip-Hop','Jazz','Metal','New Age','Oldies','Other','Pop','R&B',
  		      'Rap','Reggae','Rock','Techno','Industrial','Alternative','Ska',
  		      'Death Metal','Pranks','Soundtrack','Euro-Techno','Ambient',
  		      'Trip-Hop','Vocal','Jazz-Funk','Fusion','Trance','Classical',
  		      'Instrumental','Acid','House','Game','Sound Clip','Gospel','Noise',
  		      'Alt. Rock','Bass','Soul','Punk','Space','Meditative',
  		      'Instrumental Pop','Instrumental Rock','Ethnic','Gothic',
  		      'Darkwave','Techno-Industrial','Electronic','Pop-Folk','Eurodance',
  		      'Dream','Southern Rock','Comedy','Cult','Gangsta Rap','Top 40',
  		      'Christian Rap','Pop-Funk','Jungle','Native American','Cabaret',
  		      'New Wave','Psychedelic','Rave','Showtunes','Trailer','Lo-Fi',
  		      'Tribal','Acid Punk','Acid Jazz','Polka','Retro','Musical',
  		      'Rock&Roll','Hard Rock','Folk','Folk-Rock','National Folk',
		      'Swing','Fast-Fusion','Bebob','Latin','Revival','Celtic',
  		      'Bluegrass','Avantgarde','Gothic Rock','Progressive Rock',
  		      'Psychedelic Rock','Symphonic Rock','Slow Rock','Big Band',
  		      'Chorus','Easy Listening','Acoustic','Humour','Speech','Chanson',
  		      'Opera','Chamber Music','Sonata','Symphony','Booty Bass','Primus',
  		      'Porn Groove','Satire','Slow Jam','Club','Tango','Samba',
  		      'Folklore','Ballad','Power Ballad','Rhythmic Soul','Freestyle',
  		      'Duet','Punk Rock','Drum Solo','Acappella','Euro-House',
  		      'Dance Hall','Goa','Drum&Bass','Club-House','Hardcore','Terror',
  		      'Indie','BritPop','Negerpunk','Polsk Punk','Beat',
  		      'Christian Gangsta Rap','Heavy Metal','Black Metal','Crossover',
  		      'Contemporary Christian','Christian Rock','Merengue','Salsa',
  		      'Thrash Metal','Anime','JPop','Synthpop');

	my ($mp3info, $id3);

	#initialise all hash keys to avoid undefined error msgs in primecheck
	#when mp3s don't have certain fields

	$mp3info{ARTIST}="n/a";
	$mp3info{ALBUM}="n/a";
	$mp3info{GENRE}="n/a";
	$mp3info{BITRATE}="n/a";
	$mp3info{FREQUENCY}="n/a";
	$mp3info{MODE}="n/a";
	$mp3info{COMMENT}="n/a";
	$mp3info{YEAR}="n/a";
	$mp3info{MPEG}="n/a";
	$mp3info{LAYER}="n/a";

	open (MP3, "< $filename") || die("couldn't open $filename");

	# read MP3 header
	read(MP3, $mp3info, 4);

	# cope with an ID3v2 tag being at the start
	if (substr($mp3info, 0, 3) eq 'ID3') {
		read(MP3, $mp3info, 6, 4);
		my (undef, $major, $minor, $flags, $id3size) = unpack("a3CCB8N", $mp3info);
		$id3size = id3v2_size($id3size) + (substr($flags, 3, 1) ? 10 : 0);
		read(MP3, $mp3info, $id3size);
		read(MP3, $mp3info, 4);
	}

	# read ID3 tag
	seek(MP3, -128, 2);
	read(MP3, $id3, 128);
	close(MP3);
	# parse header info
	my $bits = unpack("B*", $mp3info);

	# unpack the parts
	$mp3info{MPEG} = $mpeg[substr($bits, 12, 1)];
	$mp3info{LAYER} = $layer[bin2dec(substr($bits, 13, 2))];
	$mp3info{BITRATE} = $bitrate{"$mp3info{MPEG}:$mp3info{LAYER}"}->[bin2dec(substr($bits, 16, 4))];
	$mp3info{FREQUENCY} = $frequency[bin2dec(substr($bits, 20, 2))];
	$mp3info{MODE} = $mode[bin2dec(substr($bits, 24, 2))];
	if (($mp3info{BITRATE}) and ($mp3info{BITRATE} =~ /(\d+)/)) {
	    my $length = ($filesize * 8) / $mp3info{BITRATE} / 1000;
	    $mp3info{MM} = $length / 60;
	    $mp3info{SS} = $length % 60;
	} else {
	    $mp3info{MM} = 0;
	    $mp3info{SS} = 0;
	}
	$mp3info{LENGTH} = sprintf "%.2d:%.2d",$mp3info{MM},$mp3info{SS};

	# does it have an ID3 tag
	my @names = qw(TAG SONGNAME ARTIST ALBUM YEAR COMMENT GENRE);
	if (substr($id3, 0, 3) eq 'TAG') {
	    # parse ID3 tag
	    my @fields = unpack("a3a30a30a30a4a30C", $id3);
	    # unpack the parts
	    for (my $i = 1; $i <= $#fields; $i++) {
		$fields[$i] =~ s/^(.*?)\s*?\000.*$/$1/;
		$mp3info{$names[$i]} = $fields[$i];
	    }
	    $mp3info{GENRE} = $genres[$mp3info{GENRE}] || 'Unknown';
	} else {
	    for (my $i = 1; $i <= $#names; $i++) {
	    	$mp3info{$names[$i]} = "unknown";
	    }
	}
    return %mp3info;
}
