#!/usr/bin/perl -w
#  ^---- you maybe have to change this path
##############################################################################
#                                                                            #
#  primeNUKE v2.2 (c) 2000 primemover                                        #
#                                                                            #
#  Nuke / Unnuke script for OpenFTPD                                         #
#                                                                            #
#  Features: * announce the nuke to an eggdrop bot (optional)                #
#            * create a info directory like "Nuked_1x_by_root_(crap)"        #
#            * delete the whole directory if flag was set                    #
#            * writes nukeinfo into nukes.log                                #
#  ------------------------------------------------------------------------  #
#                                                                            #
#  This program is free software; you can redistribute it and/or modify      #
#  it under the terms of the GNU General Public License as published by      #
#  the Free Software Foundation; either version 2 of the License, or         #
#  (at your option) any later version.                                       #
#                                                                            #
##############################################################################

use strict; # always
use FindBin;
use lib "$FindBin::Bin/";
use primetools;
use Cwd;
use File::Basename;
use File::Path;

my $debug = 0;

my ($dir, $sf, $statusfilepath, @statusfiles);
my $standarddir = $ENV{'F4A_STD'} or die "Can't find OPENFTPD directory!\n";
my $nukelog = "nuke.log";
my $nukelist = $standarddir."var/log/nukes.log";
our %nukees=();
our %registry = read_registry();
our ($nukername,$nukergroup,$time,$factor,$reason);
our ($usr,$grp,$size,$filename, $all_nukees);
our $totalsize = 0;
our $totalfiles = 0;
our $section = $ENV{'F4A_SECTION'};
our $directory = basename($ARGV[1]);
our ($fmtfile_nuke, $fmtfile_nukee,$fullpath,$announcefile,$botpidfile,$header);


# ----------------------------------------------------------------------------
#  calc nukees from nukelog
# ----------------------------------------------------------------------------
sub calc_nukees {
   open(NUKELOG, "< $_[0]/$nukelog") or die "unable to open $nukelog\n";
   $header = <NUKELOG>;
   if ($header =~ /nuked by (.+)\/(.+) at (\d{4}-\d\d-\d\d \d\d:\d\d:\d\d) factor: (.+) reason: (.+)/) {
      $nukername  = $1;
      $nukergroup = $2;
      $time	  = $3;
      $factor     = $4;
      $reason     = $5;  
   }
   else { return 0; }
   while (<NUKELOG>) {
      chomp($_);
      if ($_ =~ /(.+)\s+(.+)\s+(\d+)\s+(.+)/) {
         ($usr,$grp,$size,$filename) = (split(/\s+/, $_));
         if (not exists $nukees{$usr}) {
            $nukees{$usr}{group} = $grp;
            # don't add to filecounter if empty
	    if ($filename ne "-empty-") {
	       $nukees{$usr}{files} = 1;
            } else {
	       $nukees{$usr}{files} = 0;
	    }
	    $nukees{$usr}{size}  = $size;
         } else {
            $nukees{$usr}{files} ++;
            $nukees{$usr}{size} += $size;
         }
         if ($filename ne "-empty-") {
            $totalsize += $size;
            $totalfiles++;
         } else {
	    $totalsize = $size;
	 }
      }	 
   }
   # Write nukes-file
   if ($ARGV[0] eq "-n") {
	open (NUKELIST, ">>$nukelist");
	#time, section, releasename, nuker, nukergroup, nukee, nukeegroup, files, size, factor, reason
	foreach my $nukee (keys %nukees) {
	    printf NUKELIST "$time;$section;$directory;$nukername;$nukergroup;$nukee;$nukees{$nukee}{group};$nukees{$nukee}{files};$nukees{$nukee}{size};$totalsize;$factor;$reason\n";
	}
	close(NUKELIST);
   }
   elsif ($ARGV[0] eq "-u") {
	my $newdir=$directory;
	$newdir=~ s/^_Nuked_//i;
	print "$newdir\n";
	open (NUKELIST, "<$nukelist");
	open (NEWNUKELIST, ">$nukelist".".new");
	while (<NUKELIST>) {
	    if ($_ =~ /$newdir/i) { 
		next;
	    } else {
		printf NEWNUKELIST "$_"; 
	    }
	}
	close(NEWNUKELIST);
	close(NUKELIST);
	rename ("$nukelist.new", "$nukelist") or die "Can't update $nukelist";
   }
   close(NUKELOG);
   return 1;
}

# ----------------------------------------------------------------------------
#   announce_nuke() 
# ----------------------------------------------------------------------------
sub announce_nuke {
	my $one_nukee;
	my @rules;

	#and get formatting
	open(FORMATFILE, "< $fmtfile_nuke") or 
		die "ERROR: $fmtfile_nuke not found.\n";
	my $fmtstr_nuke = <FORMATFILE>;
	close(FORMATFILE);
	chomp($fmtstr_nuke);
	open(FORMATFILE, "< $fmtfile_nukee") or
		die "ERROR: $fmtfile_nukee not found.\n";
	my $fmtstr_nukee = <FORMATFILE>;
	close(FORMATFILE);
	chomp($fmtstr_nukee);

	# nukestats on a single line
	foreach my $nukee (sort {$nukees{$b}{size} <=> $nukees{$a}{size}} keys %nukees) {
		# printf "%s/%s (%s) ", $nukee, $nukees{$nukee}{group}, $nukees{$nukee}{size};
		$size = filesize_to_string($nukees{$nukee}{size});
		@rules = (
			[ '$username'		, "$nukee"			],
			[ '$groupname'		, "$nukees{$nukee}{group}"	],
			[ '$files'		, "$nukees{$nukee}{files}"	],
			[ '$size'		, "$size"			]
		);
		$one_nukee = substitute($fmtstr_nukee,0,@rules);
		$all_nukees .= $one_nukee . " ";
	}

	if ($totalfiles > 0) {
		$totalsize = filesize_to_string($totalsize);
	}
	else {
		$totalfiles="No";
		$totalsize = filesize_to_string(0);
	}
	@rules = (
		[ '$sitename'		, "$registry{shortsitename}"	],
		[ '$directory'		, "$directory"			],
		[ '$nukername'		, "$nukername"			],
		[ '$nukergroup'		, "$nukergroup"			],
		[ '$reason'		, "$reason"			],
		[ '$factor'		, "$factor"			],
		[ '$totalsize'		, "$totalsize"			],
		[ '$totalfiles'		, "$totalfiles"			],
		[ '$nukees'		, "$all_nukees"			],
		[ '$time'		, "$time"			]
	);
	if($announcefile and $botpidfile) {
		announce_line($announcefile, $botpidfile, substitute($fmtstr_nuke,0,@rules));
	}
}

###############################################################################

if (($#ARGV != 3) or ($ARGV[0] !~ /-(u|n)/i)) {
	print "Wrong number of arguments or wrong syntax.\n";
	exit -1;
}

#Get command line variables
	if ($ARGV[0] eq "-n") {
		$fmtfile_nuke  = $standarddir . "msg/irc/nuke";
		$fmtfile_nukee = $standarddir . "msg/irc/nukee";
		$fullpath = dirname($ARGV[1]) . "/" . $ARGV[2] . basename($ARGV[1]);
	} 
	elsif ($ARGV[0] eq "-u") {
		$fmtfile_nuke  = $standarddir . "msg/irc/unnuke";
		$fmtfile_nukee = $standarddir . "msg/irc/unnukee";
		$fullpath = $ARGV[1];
		$fullpath =~ s/\Q$ARGV[2]\E//;
	}


if (exists $registry{botpidfile}) { 
	$botpidfile = $registry{botpidfile}; 
}
if (exists $registry{announcefile}) { 
	$announcefile = $registry{announcefile}; 
}

if (calc_nukees($fullpath)) { 
	if (defined $botpidfile and defined $announcefile) {
		announce_nuke(); 
	}
}

# try to remove old status-files/links on nuke...
if ($ARGV[0] eq "-n") {
	$dir = basename($ARGV[1]);
	$statusfilepath = $fullpath . "/..";		#TODO read statusfilepath from section.cfg ?
	if (opendir(DIR, $statusfilepath)) {
		$dir =~ s/\s/_/g;
		@statusfiles = grep {/-.*-\[.+\]-\[.+\]-\[\Q$dir\E(_\(.+\)|)\]-/} readdir(DIR);
		closedir(DIR);
		foreach $sf (@statusfiles) {
			if($debug) { print " Removing $statusfilepath/$sf\n"; }
			unlink($statusfilepath . "/" . $sf);
		}
	}
}


if ($ARGV[0] eq "-n") {
	if ($ARGV[3]) {
	# just make sure we are inside the site tree and don't delete
	# something wrong 
		if ($fullpath =~ /^$registry{basedir}.+/) {
			rmtree($fullpath);
		}
	} else {
	# create statusdir
		$reason =~ s/\s/_/g;
		mkdir("$fullpath/Nuked_${factor}x_by_${nukername}_($reason)", 0700);
	}
}
elsif ($ARGV[0] eq "-u") {
	opendir(DIR, $fullpath);
	my @junk = grep { /Nuked_.+/i } readdir(DIR);
	closedir(DIR);
	foreach my $d (@junk) {
		rmdir("$fullpath/$d") or print "Unable to remove $d\n";
	}
	unlink("$fullpath/$nukelog");
}

# --- end of file ---
