#!/usr/bin/perl
##############################################################################
#                                                                            #
#  NUKES v0.1 (c) 2001 pedro                                                 #
#                                                                            #
#  show the latest nukes on your OpenFTPD site                               #
# ---------------------------------------------------------------------------#
#                                                                            #
#  This program is free software; you can redistribute it and/or modify      #
#  it under the terms of the GNU General Public License as published by      #
#  the Free Software Foundation; either version 2 of the License, or         #
#  (at your option) any later version.                                       #
#                                                                            #
##############################################################################
# Done:                                                                      #
#  - shows Nukes :)                                                          #
#  - Parameters "-l" and "-s" are working to                                 #
#  - atm only "-s" OR "-u" is working NOT both together                      #
#                                                                            #
# ToDo:                                                                      #
#  - Fix "-u" parameter                                                      #
#  - More testing                                                            #
#                                                                            #
##############################################################################
use Cwd;
use Time::Local;
use File::Basename;
use File::Find;
use Getopt::Std;

use FindBin;
use lib "$FindBin::Bin/";
use primetools;

$standarddir   = $ENV{'F4A_STD'};
# ----------------------------------------------------------------------------
#  the directory var/log/nukes file
# ----------------------------------------------------------------------------
$nukelist = $standarddir."var/log/nukes.log";

# ----------------------------------------------------------------------------
#  default number of lines
# ----------------------------------------------------------------------------
$lines = 10;

# ----------------------------------------------------------------------------
# Format Section
# ----------------------------------------------------------------------------
# You can use these variables:
#       $time = time and date when release was nuke
#    $section = the section of nuked release
#  $directory = name of nuked directory
#      $nuker = the guy who nuked that
# $nukergroup = the group the nuker is in
#      $nukee = the guy who was nuked
# $nukeegroup = the group the nukee is in
#      $files = how many file got nuked
#       $size = size of nuke
#  $totalsize = the totalsize of the nuked release
#     $factor = nukefactor
#     $reason = the reason why release got nuked

format HEADER = 
!e.--------------------------------------------------------------------------.!0
!e|!H  openFTPD NUKES 0.1                                                      !e|!0
!e|--------------------------------------------------------------------------|!0
.

format BODY =
!e|[!DAGE: !H@>>>>>>>>!e]-[!H@|||||||!e]-[!H@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<!e]|!0
        $age,      $section,  $directory,
!e|[!DNUKEE:!H@>>>>>>>!e]-[!DNUKER:!H@>>>>>>>!e]-[!H@<!Dx!H@>>>>>>>!e]-[!DReason: !H@<<<<<<<<<<<<<<<!e]|!0
         $nukee,          $nuker,    $factor,$size         $reason
!e|                                                                          |!0
.

# here's the nukelist as it is shown in glftpd :)

#format BODY =
# |  Nuker  |  Nukee  |  Amount |   Age  |  Reason   |  Nuked Directory      |
# |---------|---------|------------------|-----------|-----------------------|
# | @<<<<<<<| @<<<<<<<|@<x@>>>>> @>>>>>>>|@<<<<<<<<<<|@<<<<<<<<<<<<<<<<<<<<<<|
#   $nuker,   $nukee,$factor,$size,$age,  $reason,    $directory
#.

format FOOTER = 
!e|--------------------------------------------------------------------------|!0
!e|!H SITE NUKES [-l <lines>] [-s <section>] [-u <username>] for filtered view.!e|!0
!e`--------------------------------------------------------------------------'!0
.
#`

# ----------------------------------------------------------------------------
#  strdifftime($earlier, $later) - calc the diff between two times as string
# ----------------------------------------------------------------------------
sub strdifftime {
   my $time1   = $_[0]; my $time2 = $_[1];
   my $diffstr = "";  my $diff = $time2 - $time1;
   my $seconds = $diff % 60; $diff = ($diff - $seconds) / 60;
   my $minutes = $diff % 60; $diff = ($diff - $minutes) / 60;
   my $hours   = $diff % 24; $diff = ($diff - $hours) / 24;
   my $days    = $diff % 7; 
   my $weeks   = ($diff - $days) / 7;

   if ($weeks != 0) { 
      $diffstr .= sprintf("%02dw", $weeks); 
      if ($days or $hours or $minutes or $seconds) {
         $diffstr .= " ";
      }
   }
   if ($days  != 0) { 
      $diffstr .= sprintf("%02dd", $days);  
      if ($hours or $minutes or $seconds) {
         $diffstr .= " ";
      }
   }
   if ($hours != 0 and $weeks == 0) { 
      $diffstr .= sprintf("%02dh", $hours); 
      if ($minutes or $seconds) {
         $diffstr .= " ";
      }	 
   }
   if ($minutes != 0 and $days == 0 and $weeks == 0) { 
      $diffstr .= sprintf("%02dm", $minutes); 
      if ($seconds) {
         $diffstr .= " ";
      }	      
   }
   return $diffstr;
}

###############################################################################
# main program                                                                #
###############################################################################

@lastnukes = ();
getopt("s:l:u:", \%args);

#$filter = ".+";
if (exists $args{s}) {
   $filter = $args{s}; 
}

if (exists $args{u}) {
   $filter = $args{u}; 
}

if (exists $args{l}) {
   $lines = $args{l}; 
}


$~ = "HEADER"; write;
open(NUKES, "< $nukelist") or die "ERROR unable to open nukes log\n";
while (<NUKES>) {
   chomp($_);
      my $section = (split(/;/, $_))[1];
      my $username = (split(/;/, $_))[5];
      if ($section =~ /$filter/i) {
	 push(@lastnukes, $_); 
      }
      elsif ($username =~ /$filter/i) {
	 push(@lastnukes, $_); 
      }

      if ($#lastnukes >= $lines) { 
	 shift(@lastnukes); 
      }   
}
close(NUKES);

$number = 0;

foreach my $nuke (reverse @lastnukes) {
   $time       = 0; $section    = 0; $directory = 0; $nuker     = 0; 
   $nukergroup = 0; $nukee      = 0; $nukeegroup= 0; $files     = 0; 
   $size       = 0; $totalsize  = 0; $factor    = 0; $reason    = 0;
   ($time, $section, $directory, $nuker, $nukergroup, $nukee, $nukeegroup,
    $files, $size, $totalsize, $factor, $reason)=split(";",$nuke);
   $size=filesize_to_string($size);      
   $tm = stringtotime($time); 
   $age = strdifftime($tm, time);
   $~ = "BODY"; write;
     
}
$~ = "FOOTER"; write;

# --- end of file ---
