#!/usr/bin/perl -w

##############################################################################
#                                                                            #
#  OpenFTPD PRE script v1.0 (c) 2000 topace                                  #
#                                                                            #
#  ------------------------------------------------------------------------  #
#                                                                            #
#  This program is free software; you can redistribute it and/or modify      #
#  it under the terms of the GNU General Public License as published by      #
#  the Free Software Foundation; either version 2 of the License, or         #
#  (at your option) any later version.                                       #
#                                                                            #
##############################################################################

use strict; #always
use FindBin;
use lib "$FindBin::Bin/";
use primetools;
use Cwd;
use Cwd 'abs_path';
use File::Basename;
use File::Path;
use File::Find;


# shared vars :
our ($botpidfile,$announcefile,$ftpbasedir,$sitename,$username,$groupname,
     $formatfile_pre, $directory, $presection,$prepath,$numfiles,$goodfiles,
     $grouppath,$releasesize);

# ----------------------------------------------------------------------------
#   announce_pre() 
# ----------------------------------------------------------------------------
sub announce_pre {
   my $formatstring;
   my @rules;
   
   open(FORMATFILE, "< $formatfile_pre") or 
      die "ERROR: $formatfile_pre not found.\n";
   $formatstring = <FORMATFILE>;
   close(FORMATFILE);
   chomp($formatstring);

	@rules = (
		[ '$sitename'		, "$sitename"		],
		[ '$section'		, "$presection"		],
		[ '$directory'		, "$directory"		],
		[ '$username'		, "$username"		],
		[ '$groupname'		, "$groupname"		],
		[ '$numfiles'		, "$numfiles"		],
		[ '$releasesize'	, "$releasesize"	]
	);
	if($announcefile and $botpidfile) {
		announce_line($announcefile, $botpidfile, substitute($formatstring,0,@rules));
	}
}

# From Primetools, a hacked down version of timetostring 
# ----------------------------------------------------------------------------
#   timetostring() - returns string
#   create a string in YYYY-MM-DD HH:MM:SS format from the current time
# ----------------------------------------------------------------------------
# uhm, we include primetools, so why not use it?
#sub timetostring {
#
#   ($seconds, $minutes, $hours, $day_of_month, $month, $year) = (localtime)[0..5];
#   $timestring = sprintf("%04d-%02d-%02d %02d:%02d:%02d",  $year+1900, $month+1,
#   $day_of_month, $hours, $minutes, $seconds);
#   return $timestring;
#}


###############################################################################

if ($#ARGV != 1) {
   print "!HSyntax: !cSITE PRE !C<!csection!C> !C<!cdirectory!C>!0\n",
         "!HPre <directory> into <section>!0\n";
   exit 0;
}

my %registry = read_registry();
$directory = basename($ARGV[1]);

if (exists $registry{botpidfile}) { 
   $botpidfile = $registry{botpidfile}; 
}
if (exists $registry{announcefile}) { 
   $announcefile = $registry{announcefile}; 
}
if (exists $registry{basedir}) { 
   $ftpbasedir = $registry{basedir}; 
}

if (exists $registry{shortsitename}) { 
   $sitename = $registry{shortsitename}; 
}
else { $sitename = "XYZ" } 
if (exists $ENV{'F4A_USER'}) { $username = $ENV{'F4A_USER'} }
   else { $username = "unknown" };
if (exists $ENV{'F4A_GROUP'}) { $groupname = $ENV{'F4A_GROUP'} }
   else { $groupname = "unknown" };

$formatfile_pre  = $standarddir . "msg/irc/pre";
my $sectioncfg      = $standarddir . "etc/sections.cfg";
my $directorylog    = $standarddir . "var/log/directory.log";

my $cwd=getcwd();
my $cwdir=basename($cwd);

if( ! -d $directory ) {
  print "Can't find directory ($directory) to PRE!\n";
  exit 0;
}

my $matched=0;
# now open the sections.cfg file, and grab all the path names of active sections
# also make sure there is a GROUPS section, and a groups directory defined
  open(SECCFG,"< $sectioncfg") or
     die "ERROR: unable to open sectioncfg";
  while(<SECCFG>) {
    if($_ =~ /[#].*/) {
      #ignore cuz its a comment
    }
    else
    {
      chomp($_);
      my ($secname,$seccfg,$secstat,$secpath) = split(/\s+/,$_,4);
      if($ARGV[0] =~ /^$secname$/i) {
        $presection = $secname;
        $prepath = $secpath;
        $matched++;
      }
      if("GROUPS" =~ /^$secname$/i) {
        my $groupsection = $secname;
        $grouppath = $secpath;
        $matched++;
      }
    }
  }
  close(SECCFG);

  my $groupdir = "$ftpbasedir$grouppath\/$groupname";

  if( ! -d $groupdir ) {
    print "You don't seem to have a group directory! ($grouppath\/$groupname)\n";
    exit 0;
  }

  $groupdir = abs_path($groupdir);

 #mow make sure they are in their OWN groups directory and inside the ftpsite tree (security)
 if($cwd !~ /^$groupdir$/) {
   print "You can only pre from YOUR GROUPS DIRECTORY! ($grouppath\/$groupname)\n";
   exit 0;
 }

  #matched will equal two if and only if the pre section and groups section were found
  if( $matched eq 2) { 
    # All good, go ahead and make it happen!
    print "Found section and directory!  Pre'ing $directory to $prepath\n"; 

    # first lets get some release info tho, #files, size.
    ($goodfiles,$numfiles)=("unknown","unknown");
    if (open (STATUSFILE, "< $directory/.status")) { 
      chomp($_=<STATUSFILE>);
      my ($goodfiles,$numfiles) = split (/\s+/,$_,2); 
      close (STATUSFILE);
    }

    $releasesize=0;
    find sub { $releasesize += -s }, $directory; 
    $releasesize = sprintf("%4.1f",($releasesize/1048576));

    system ('mv', $cwd.'/'.$directory, $ftpbasedir.$prepath); 
#    `cp -rf $cwd/$directory $ftpbasedir$prepath`; 

# If the source and destination are on the SAME filesystem, you can leave this how it is, if not,
# some brands of *nix might not allow a mv across filesystems, so switch the above line to the
# cp one, and then uncomment the code below if you want the source removed

   #  if ($cwd =~ /^$registry{basedir}.+/) {
   #     rmtree("$cwd/$directory");
   #  } #end if 
    
    # Announce the PRE
    if (defined $botpidfile and defined $announcefile) {
       announce_pre(); 
    }

    # Add it to the directory.log so it shows up in `site new`
     if (-e $directorylog) {
         open(DIRLOG, ">> $directorylog") or die "ERROR: unable to open directory logfile.\n";
      }
      else {
         open(DIRLOG, "> $directorylog") or die "ERROR: unable to open directory logfile.\n";
      }
      my $relpath=$prepath."/".$directory;
      printf DIRLOG "%s %-10s %-10s %-16s %s\n", timetostring(), $username, $groupname, $presection, $relpath;
      close(DIRLOG);

  } # end if
  else
  {
    print "Couldnt find section ($ARGV[0]), you must enter a valid section to Pre INTO\n";
    exit 0;
  }


# --- end of file ---
