#!/usr/bin/perl -w

##############################################################################
#                                                                            #
#  statfilter ... will show you inactive users by parsing the statfiles      #
#  (c) 2002 primemover                                                       #
#                                                                            #
#  Syntax: statfilter.pl <u|g><d|w|m|y|t><u|d> [section] [minimum size]      #
# ---------------------------------------------------------------------------#
#                                                                            #
#  This program is free software; you can redistribute it and/or modify      #
#  it under the terms of the GNU General Public License as published by      #
#  the Free Software Foundation; either version 2 of the License, or         #
#  (at your option) any later version.                                       #
#                                                                            #
##############################################################################

$debug = 0;

##############################################################################

use FindBin;
use lib "$FindBin::Bin/";
use primetools;
use Cwd;
use File::Basename;

sub load_stats {
   $statfile = "$standarddir/$statdir/" . lc($section) . ".txt";
   if ($debug) { print "DEBUG: $statfile\n"; }
   open(STATFILE, "< $statfile") or return 0;
   if ($debug) { print "DEBUG: statfile opened sucessfully\n"; }
   while (<STATFILE>) {
      if ($debug) { print "DEBUG: 1 line read from statfile\n"; }
      @s = split(/\t/, $_);
      if ($#s != 33) {
         die "unknown file format.";
      }
      $username = $s[0];
      $stats{$username}{userinfo}    = $s[1];
      $stats{$username}{groupname}   = $s[2];
      $stats{$username}{groupinfo}   = $s[3];
      $stats{$username}{dl_f_day}    = $s[4];
      $stats{$username}{dl_day}      = $s[5];
      $stats{$username}{dl_tm_day}   = $s[6];
      $stats{$username}{ul_f_day}    = $s[7];
      $stats{$username}{ul_day}      = $s[8];
      $stats{$username}{ul_tm_day}   = $s[9];
      $stats{$username}{dl_f_week}   = $s[10];
      $stats{$username}{dl_week}     = $s[11];
      $stats{$username}{dl_tm_week}  = $s[12];
      $stats{$username}{ul_f_week}   = $s[13];
      $stats{$username}{ul_week}     = $s[14];
      $stats{$username}{ul_tm_week}  = $s[15];
      $stats{$username}{dl_f_month}  = $s[16];
      $stats{$username}{dl_month}    = $s[17];
      $stats{$username}{dl_tm_month} = $s[18];
      $stats{$username}{ul_f_month}  = $s[19];
      $stats{$username}{ul_month}    = $s[20];
      $stats{$username}{ul_tm_month} = $s[21];
      $stats{$username}{dl_f_year}   = $s[22];
      $stats{$username}{dl_year}     = $s[23];
      $stats{$username}{dl_tm_year}  = $s[24];
      $stats{$username}{ul_f_year}   = $s[25];
      $stats{$username}{ul_year}     = $s[26];
      $stats{$username}{ul_tm_year}  = $s[27];
      $stats{$username}{dl_f_total}  = $s[28];
      $stats{$username}{dl_total}    = $s[29];
      $stats{$username}{dl_tm_total} = $s[30];
      $stats{$username}{ul_f_total}  = $s[31];
      $stats{$username}{ul_total}    = $s[32];
      $stats{$username}{ul_tm_total} = $s[33];
   }
   close(STATFILE);
   return 1;
}

sub print_user_stats {
   my $v1 = $_[0] . '_' . $_[1];
   my $v2 = $_[0] . '_tm_' . $_[1];
   my $v3 = $_[0] . '_f_' . $_[1];
   my $rank = 0;
   foreach $user (keys %stats) {   
      my $mbsize = $stats{$user}{$v1} / 0x100000;
      my $speed = $stats{$user}{$v2} > 0 ? 
                  $stats{$user}{$v1} / ($stats{$user}{$v2} * 1024) : 0;
      my $files = $stats{$user}{$v3};
      my $group = $stats{$user}{groupname};
      my $info  = $stats{$user}{userinfo};
      $rank++;
      if ($mbsize < $minmb) { 
      	  print "$user ($group) is too inactive - " . sprintf("%.1f", $mbsize) . " mb.\n";
      }
   }
}

# ----------------------------------------------------------------------------
#  print_group_stats(<ul|dl>, <day|week|month|year|total>)
# ----------------------------------------------------------------------------
sub print_group_stats {
   my %groupstats = ();
   my $v1 = $_[0] . '_' . $_[1];
   my $v2 = $_[0] . '_tm_' . $_[1];
   my $rank = 0;
   foreach $user (keys %stats) {
      my $groupname = $stats{$user}{groupname};
      if (exists $groupstats{$groupname}) {
         $groupstats{$groupname}{$v1} += ($stats{$user}{$v1});
         $groupstats{$groupname}{$v2} += ($stats{$user}{$v2});
      }
      else {
	 $groupstats{$groupname}{groupinfo} = $stats{$user}{groupinfo};
         $groupstats{$groupname}{$v1} = $stats{$user}{$v1};
         $groupstats{$groupname}{$v2} = $stats{$user}{$v2};
      }
   }
   foreach $group (sort {$groupstats{$b}{$v1} <=> $groupstats{$a}{$v1}} keys %groupstats) {
      my $mbsize = $groupstats{$group}{$v1} / 0x100000;
      my $speed = $groupstats{$group}{$v2} > 0 ? 
                  $groupstats{$group}{$v1} / ($groupstats{$group}{$v2} * 1024) : 0;
      my $info = $groupstats{$group}{groupinfo};
      $rank++;
      if ($mbsize < $minmb) { 
            print "$group is inactive - " . sprintf("%.1f", $mbsize) . " mb.\n";
      }
   }
}


sub refresh_stats {
   open(PIDFILE, "< ${standarddir}/var/ftpd.pid");
   $pid = <PIDFILE>;
   chomp($pid);
   kill USR1 => $pid;
   sleep(1);
}

###############################################################################
# main program                                                                #
###############################################################################

$statdir = "var/stats";
if (exists $ENV{'F4A_STD'}) {
  $standarddir = $ENV{'F4A_STD'};
}
elsif ( -e "$ENV{'HOME'}/.openftpdrc" ) { 
  $standarddir = `cat $ENV{'HOME'}/.openftpdrc`;
  $standarddir =~ s/\n//;
  $standarddir =~ s/\r//;
}
else {
  print "ERROR: Cant locate your standarddir either in Environment F4A_STD or in $ENV{'HOME'}/.openftpdrc\n"; 
  exit 1;
}

%stats = ();
$minmb = 1024;  # default minimum 1 gb 

if (exists $ENV{'F4A_SECTION'}) { 
   $section = lc($ENV{'F4A_SECTION'});
}
else { 
   # couldnt read the section, so lets use global, instead of exiting...
   $section = "global";
#   print "ERROR: Unable to read F4A_SECTION\n";
#   exit 1;
};

# we want to see the global stats and not the default one
if ($section eq "default") {
   $section = "global";
}

if ($#ARGV < 0 || $#ARGV > 2) {
   print "OpenFTPD statfilter script (c) 2000 primemover\n";
   print "Syntax: statfilter.pl <u|g><d|w|m|y|t><u|d> [section] [minimum mb]\n";
   print "        Example: stats.pl uwu iso 10\n";
   exit;
}

if ($#ARGV > 0) {
   $section = $ARGV[1];
   chomp($ARGV[1]);
}

if ($#ARGV > 1 && $ARGV[2] =~ /\d+/) {
   $minmb = $ARGV[2];
}
refresh_stats();
load_stats();

if ($ARGV[0] =~ /(u|g)(d|w|m|y|t)(u|d)/) {
  if ($1 eq "u") {
     if ($2 eq "d") {
        if ($3 eq "u") {
           print_user_stats("ul", "day");	
	}
	elsif ($3 eq "d") {
           print_user_stats("dl", "day");
	}
     }
     elsif ($2 eq "w") {
        if ($3 eq "u") {
           print_user_stats("ul", "week");
	}
	elsif ($3 eq "d") {
           print_user_stats("dl", "week");
	}
     }
     elsif ($2 eq "m") {
        if ($3 eq "u") {
           print_user_stats("ul", "month");
	}
	elsif ($3 eq "d") {
           print_user_stats("dl", "month");
	}    
     }
     elsif ($2 eq "y") {
        if ($3 eq "u") {
           print_user_stats("ul", "year");
	}
	elsif ($3 eq "d") {
           print_user_stats("dl", "year");
	}     
     }
     elsif ($2 eq "t") {
        if ($3 eq "u") {
           print_user_stats("ul", "total");	
	}
	elsif ($3 eq "d") {
           print_user_stats("dl", "total");
	}    
     }
  }
  elsif ($1 eq "g") {
     if ($2 eq "d") {
        if ($3 eq "u") {
           print_group_stats("ul", "day");
	}
	elsif ($3 eq "d") {
           print_group_stats("dl", "day");
	}
     }
     elsif ($2 eq "w") {
        if ($3 eq "u") {
           print_group_stats("ul", "week");	
	}
	elsif ($3 eq "d") {
           print_group_stats("dl", "week");
	}
     }
     elsif ($2 eq "m") {
        if ($3 eq "u") {
           print_group_stats("ul", "month");
	}
	elsif ($3 eq "d") {
           print_group_stats("dl", "month");
	}    
     }
     elsif ($2 eq "y") {
        if ($3 eq "u") {
           print_group_stats("ul", "year");
	}
	elsif ($3 eq "d") {
           print_group_stats("dl", "year");
	}     
     }
     elsif ($2 eq "t") {
        if ($3 eq "u") {
           print_group_stats("ul", "total");	
	}
	elsif ($3 eq "d") {
           print_group_stats("dl", "total");
	}    
     }
   }  
}

# --- end of file ---
