# OpenFTPD SITE NEW script (shows last directories)
# (c) 2001 primemover / team openftpd

my $directorylog = "$FTP::cfg_var_path/log/directory.log";

# ----------------------------------------------------------------------------
#  show uploads that were already deleted ?
# ----------------------------------------------------------------------------
my $showdeleted   = 0;

# ----------------------------------------------------------------------------
#  ignore all directories matching this regexpr
# ----------------------------------------------------------------------------
my $ignore = 'cd[0-9]$|disc[0-9]|sample|cover|sbc-stats|stats|ac3-codec';

# ----------------------------------------------------------------------------
#  subdirs to count if release has more cds
# ----------------------------------------------------------------------------
my $cdmatch = 'cd[0-9]$|disc[0-9]';

# ----------------------------------------------------------------------------
#  default number of lines
# ----------------------------------------------------------------------------
my $lines = 10;

my $statusfile  = '.status';

my $body1 = 
"!h%-44.44s!G|!h%-9.9s!G|!h%3.3s!G|!h%-9.9s!G|!h%2.2s!A%c!h%2.2s!G|!h%4d!0\n";

my $body2 = 
"!H%-44.44s!G|!H%-9.9s!G|!H%3.3s!G|!H%-9.9s!G|!H%2.2s!A%c!H%2.2s!G|!H%4d!0\n";

my $header = &open_format("$FTP::cfg_msg_path/new_top");

my $footer = &open_format("$FTP::cfg_msg_path/new_tail");

#######################################################################

use Cwd;
use Time::Local;
use File::Basename;
use File::Find;

use FindBin;
use lib "$FindBin::Bin/../lib";
use utils;


my $_site_new = sub {
	my %registry = read_registry();
	my @lastdirs = ();
	my $bodydata = "";

	my $args = shift;	

	my $filter = ".+";

	if ($args =~ /-l (\d+)/i) {
		$lines = $1;	
	}

	if ($args =~ /-S (\w+)/i) {
		$filter = $1;	
	}

	open(DIRLOG, "< $directorylog") or return (200, "The log is still empty.");
	while (<DIRLOG>) {
   		chomp($_);
#   		if($_ and $_ !~ /$ignore/i) {		#this line only ignores the ignore thingy - the next only shows dirs with a - and min. 10chars
   		if($_ and $_ !~ /$ignore/i && basename($_) =~ /-/ && length(basename($_))>10) { 
      			my $section = (split(/\s+/, $_, 6))[4];
      			if ($section =~ /$filter/i) {
		 		push(@lastdirs, $_); 
      			}
      			if ($#lastdirs >= $lines) { 
		 		shift(@lastdirs); 
      			}   
   		}
	}
	close(DIRLOG);
	
	my $number = 0;
	my $i = 1;
	
	foreach my $lastdir (reverse @lastdirs) {
		my $totalsize  = 0;
		my $filecount  = 0;
		my $totalfiles = 0;
		my $goodfiles  = 0;
		my $numdirs    = 0;
		
		my ($dummy,$DIR);
   
		my ($username, $section, $directory) = (split(/\s+/, $lastdir, 6))[2, 4, 5];
		my $tm = stringtotime($lastdir); 
		my $age = strdifftime($tm, time);
		my $fullpath = $registry{basedir} . '/' . $directory;

		if ( -d $fullpath or $showdeleted) {
			$number++;
			if ( -d $fullpath) {
				$totalsize = 0;
				$numdirs = 0;
				find sub { $totalsize += -s }, $fullpath;
				if (open(STATUSFILE, "< $fullpath/$statusfile")) {
					my $s = <STATUSFILE>;
					chomp($s);
					if ($s =~ /(\d+)\s(\d+)/) {
						$goodfiles = $1;
						$totalfiles = $2;
					}
					close(STATUSFILE);
				}
				if (opendir DIR, "$fullpath") {
					while($dummy=readdir(DIR)) {
						if ($dummy =~ /($cdmatch)/i) {
					       		$dummy=$fullpath."/".$dummy;
						        -d $dummy && !-l $dummy && do { $numdirs+=1; }
						}
					}
				}
				closedir(DIR);
				
			}            
			else {
				$username = "-DELETED-"; 
			}

			my $c = $i % 2 ? 'h' : 'H';
			my $bn = substr(basename($directory), 0, 45);

			$bodydata .= sprintf($i % 2 ? $body1 : $body2, 
			                     basename($directory) . '.' x 46, 
					     $section . '.' x 9, 
			                     $age, 
		                             $username . '.' x 9, 
		                             $totalfiles == 0 ? sprintf ("%02d",$numdirs) : sprintf("%02d", $goodfiles), 
					     $totalfiles == 0 ? 32 : 47,
		                             $totalfiles == 0 ? "CD" : sprintf("%02d", $totalfiles), 
	                                     $totalsize / 0x100000);
			$i++;
		}   
	}
	return (200, $header . $bodydata . $footer);
};

sub new {
	&$_site_new(@_);
}

# bind the command
&FTP::bind_site_cmd ("new", $_site_new);
&FTP::bind_site_cmd ("last", $_site_new);

# --- end of file ---
